@@ -149,6 +149,7 @@ public:
     int GetDrawingBeamWhiteWidth(int staffSize, bool graceSize) const;
     int GetDrawingLedgerLineLength(int staffSize, bool graceSize) const;
     int GetCueSize(int value) const;
+    double GetCueScaling() const;
     ///@}
 
     Point ConvertFontPoint(const Glyph *glyph, const Point &fontPoint, int staffSize, bool graceSize) const;
@@ -187,6 +188,12 @@ public:
     double GetTopMargin(const ClassId classId) const;
     ///@}
 
+    /**
+     * Get the default distance from the staff for the object
+     * The distance is given in x * MEI UNIT
+     */
+    double GetStaffDistance(const ClassId classId, int staffIndex, data_STAFFREL staffPosition);
+
     /**
      * Prepare the MIDI timemap for MIDI and timemap file export.
      * Run trough all the layers and fill the score-time and performance timing variables.
@@ -212,8 +219,8 @@ public:
      */
     bool ExportTimemap(std::string &output);
     void PrepareJsonTimemap(std::string &output, std::map<double, double> &realTimeToScoreTime,
-        std::map<double, std::vector<std::string> > &realTimeToOnElements,
-        std::map<double, std::vector<std::string> > &realTimeToOffElements, std::map<double, int> &realTimeToTempo);
+        std::map<double, std::vector<std::string>> &realTimeToOnElements,
+        std::map<double, std::vector<std::string>> &realTimeToOffElements, std::map<double, int> &realTimeToTempo);
 
     /**
      * Set the initial scoreDef of each page.
@@ -221,17 +228,22 @@ public:
      * It uses the MusObject::SetPageScoreDef functor method for parsing the file.
      * This will be done only if m_currentScoreDefDone is false or force is true.
      */
-    void SetCurrentScoreDefDoc(bool force = false);
+    void ScoreDefSetCurrentDoc(bool force = false);
 
     /**
      * Check whether we need to optimize score based on the condense option
      */
-    bool IsOptimizationNeeded();
+    bool ScoreDefNeedsOptimization();
 
     /**
      * Optimize the scoreDef once the document is cast-off.
      */
-    void OptimizeScoreDefDoc();
+    void ScoreDefOptimizeDoc();
+
+    /**
+     * Set the GrpSym start / end for each System once ScoreDef is set and (if necessary) optimized
+     */
+    void ScoreDefSetGrpSymDoc();
 
     /**
      * Prepare the document for drawing.
@@ -246,6 +258,12 @@ public:
      */
     void CastOffDoc();
 
+    /**
+     * Casts off the entire document, only using the document's system breaks
+     * if they would be close to the end in the normal document.
+     */
+    void CastOffSmartDoc();
+
     /**
      * Casts off the entire document, using the document's line breaks,
      * but adding its own page breaks.
@@ -255,9 +273,10 @@ public:
     /**
      * Casts off the entire document, with options for obeying breaks.
      * @param useSb - true to use the sb from the document.
-     * @param usePg - true to use the pb from the document.
+     * @param usePb - true to use the pb from the document.
+     * @param smart - true to sometimes use encoded sb and pb.
      */
-    void CastOffDocBase(bool useSb, bool usePb);
+    void CastOffDocBase(bool useSb, bool usePb, bool smart = false);
 
     /**
      * Casts off the running elements (headers and footer)
@@ -310,7 +329,7 @@ public:
      * By default, the element are used only for the rendering and not preserved in the MEI output
      * Permanent conversion discard analytical markup and elements will be preserved in the MEI output.
      */
-    void ConvertMarkupDoc(bool permanent = false);
+    void ConvertMarkupDoc(bool permanent = true);
 
     /**
      * Transpose the content of the doc.
@@ -502,7 +521,7 @@ private:
 
     /**
      * A flag to indicate whether the currentScoreDef has been set or not.
-     * If yes, SetCurrentScoreDef will not parse the document (again) unless
+     * If yes, ScoreDefSetCurrent will not parse the document (again) unless
      * the force parameter is set.
      */
     bool m_currentScoreDefDone;
