@@ -31,7 +31,7 @@ public:
     {
         m_doc = doc;
         m_view = view;
-        m_editInfo = "";
+        m_editInfo.reset();
     }
     virtual ~EditorToolkit() {}
 
@@ -42,12 +42,12 @@ public:
     /**
      * Get information on the last editor function used
      */
-    virtual std::string EditInfo() { return m_editInfo; }
+    virtual std::string EditInfo() { return m_editInfo.json(); }
 
 protected:
     Doc *m_doc;
     View *m_view;
-    std::string m_editInfo;
+    jsonxx::Object m_editInfo;
 };
 } // namespace vrv
 
