@@ -63,7 +63,7 @@ enum AlignmentType {
 //----------------------------------------------------------------------------
 
 /**
- * This class stores an alignement position elements will point to
+ * This class stores an alignment position elements will point to
  */
 class Alignment : public Object {
 public:
@@ -130,7 +130,9 @@ public:
      * Returns (-)VRV_UNSET in nothing for the staff specified.
      * Uses Object::GetAlignmentLeftRight
      */
-    void GetLeftRight(int staffN, int &minLeft, int &maxRight);
+    void GetLeftRight(
+        const std::vector<int> &staffNs, int &minLeft, int &maxRight, const std::vector<ClassId> &m_excludes = {});
+    void GetLeftRight(int staffN, int &minLeft, int &maxRight, const std::vector<ClassId> &m_excludes = {});
 
     /**
      * Returns the GraceAligner for the Alignment.
@@ -149,6 +151,11 @@ public:
      */
     AlignmentReference *GetReferenceWithElement(LayerElement *element, int staffN = VRV_UNSET);
 
+    /**
+     * Return pair of max and min Y value within alignment. Elements will be counted by alignment references.
+     */
+    std::pair<int, int> GetAlignmentTopBottom();
+
     /**
      * Add an accidental to the accidSpace of the AlignmentReference holding it.
      * The Alignment has to have a AlignmentReference holding it.
@@ -178,6 +185,11 @@ public:
      */
     bool HasAlignmentReference(int staffN);
 
+    /**
+     * Return true if the alignment contains only references to timestamp attributes.
+     */
+    bool HasTimestampOnly();
+
     //----------//
     // Functors //
     //----------//
@@ -220,6 +232,11 @@ public:
      */
     virtual int AdjustAccidX(FunctorParams *functorParams);
 
+    /**
+     * See Object::AdjustDotsEnd
+     */
+    virtual int AdjustDotsEnd(FunctorParams *);
+
 private:
     /**
      * Retrieve the AlignmentReference with staffN.
@@ -263,7 +280,7 @@ private:
 
 /**
  * This class stores a references of LayerElements for a staff.
- * The staff identification (@n) is given by the attCommon and takes into accound
+ * The staff identification (@n) is given by the attCommon and takes into account
  * cross-staff situations.
  * Its children of the alignment are references.
  */
@@ -306,6 +323,11 @@ public:
      */
     bool HasMultipleLayer() const { return (m_layerCount > 1); }
 
+    /**
+     * Return true if the reference has elements from cross-staff.
+     */
+    bool HasCrossStaffElements();
+
     //----------//
     // Functors //
     //----------//
@@ -326,9 +348,9 @@ public:
     virtual int AdjustAccidX(FunctorParams *functorParams);
 
     /**
-     * See Object::UnsetCurrentScoreDef
+     * See Object::UnscoreDefSetCurrent
      */
-    virtual int UnsetCurrentScoreDef(FunctorParams *functorParams);
+    virtual int ScoreDefUnsetCurrent(FunctorParams *functorParams);
 
 private:
     //
@@ -444,6 +466,15 @@ public:
     int GetNonJustifiableMargin() const { return m_nonJustifiableLeftMargin; }
     ///@}
 
+    /**
+     * @name Set and Get the initial tstamp duration.
+     * Setter takes a meter unit parameter.
+     */
+    ///@{
+    void SetInitialTstamp(int meterUnit);
+    double GetInitialTstampDur() const { return m_initialTstampDur; }
+    ///@}
+
     /**
      * Get left Alignment for the measure and for the left BarLine.
      * For each MeasureAligner, we keep and Alignment for the left position.
@@ -517,6 +548,12 @@ private:
      * Store measure's non-justifiable margin used by the scoreDef attributes.
      */
     int m_nonJustifiableLeftMargin;
+
+    /**
+     * The time duration of the timestamp between 0.0 and 1.0.
+     * This depends on the meter signature in the preceeding scoreDef
+     */
+    double m_initialTstampDur;
 };
 
 //----------------------------------------------------------------------------
