@@ -10,6 +10,7 @@
 
 #include <cstdlib>
 #include <ctime>
+#include <functional>
 #include <iterator>
 #include <map>
 #include <string>
@@ -225,7 +226,6 @@ public:
     void SetUuid(std::string uuid);
     void SwapUuid(Object *other);
     void ResetUuid();
-    static void SeedUuid(unsigned int seed = 0);
 
     /**
      * Methods for setting / getting comments
@@ -504,6 +504,11 @@ public:
      */
     bool HasEditorialContent();
 
+    /**
+     * Return true if the object contains anything that is not editorial content
+     */
+    bool HasNonEditorialContent();
+
     /**
      * Saves the object (and its children) using the specified output stream.
      * Creates functors that will parse the tree.
@@ -532,9 +537,21 @@ public:
      * This is the generic way for parsing the tree, e.g., for extracting one single staff or layer.
      * Deepness specifies how many child levels should be processed. UNLIMITED_DEPTH means no
      * limit (EditorialElement objects do not count).
+     * skipFirst does not call the functor or endFunctor on the first (calling) level
      */
     virtual void Process(Functor *functor, FunctorParams *functorParams, Functor *endFunctor = NULL,
-        ArrayOfComparisons *filters = NULL, int deepness = UNLIMITED_DEPTH, bool direction = FORWARD);
+        ArrayOfComparisons *filters = NULL, int deepness = UNLIMITED_DEPTH, bool direction = FORWARD,
+        bool skipFirst = false);
+
+    //----------------//
+    // Static methods //
+    //----------------//
+
+    static void SeedUuid(unsigned int seed = 0);
+
+    static std::string GenerateRandUuid();
+
+    static bool sortByUlx(Object *a, Object *b);
 
     //----------//
     // Functors //
@@ -633,6 +650,15 @@ public:
     virtual int ConvertMarkupAnalyticalEnd(FunctorParams *) { return FUNCTOR_CONTINUE; }
     ///@}
 
+    /**
+     * Convert markup of artic@artic multi value into distinct artic elements.
+     * See Doc::ConvertMarkupAnalyticalDoc
+     */
+    ///@{
+    virtual int ConvertMarkupArtic(FunctorParams *) { return FUNCTOR_CONTINUE; }
+    virtual int ConvertMarkupArticEnd(FunctorParams *) { return FUNCTOR_CONTINUE; }
+    ///@}
+
     /**
      * Convert scoreDef / staffDef markup (@clef.*, @key.*) to elements.
      * See Doc::ConvertScoreDefMarkupDoc
@@ -714,6 +740,19 @@ public:
     virtual int AdjustArpegEnd(FunctorParams *) { return FUNCTOR_CONTINUE; }
     ///@}
 
+    /**
+     * Adjust the spacing for clef changes.
+     */
+    virtual int AdjustClefChanges(FunctorParams *) { return FUNCTOR_CONTINUE; }
+
+    /**
+     * Adjust the position of the dots for multiple layers
+     */
+    ///@{
+    virtual int AdjustDots(FunctorParams *) { return FUNCTOR_CONTINUE; }
+    virtual int AdjustDotsEnd(FunctorParams *) { return FUNCTOR_CONTINUE; }
+    ///@}
+
     /**
      * Adjust the position the outside articulations.
      */
@@ -742,6 +781,11 @@ public:
      */
     virtual int AdjustAccidX(FunctorParams *) { return FUNCTOR_CONTINUE; }
 
+    /**
+     * Adjust the x position of accidental.
+     */
+    virtual int AdjustTempo(FunctorParams *) { return FUNCTOR_CONTINUE; }
+
     /**
      * @name Adjust the x position of a right barline in order to make sure the is no text content
      * overlflowing in the right margin
@@ -825,6 +869,11 @@ public:
      */
     virtual int AdjustSlurs(FunctorParams *) { return FUNCTOR_CONTINUE; }
 
+    /**
+     * Adjust the position the articulations.
+     */
+    virtual int AdjustArtic(FunctorParams *) { return FUNCTOR_CONTINUE; }
+
     /**
      * Adjust the position the outside articulations with slur.
      */
@@ -929,15 +978,15 @@ public:
      * It also includes a scoreDef for each measure where a change occured before.
      * A change can be either a scoreDef before or a clef, meterSig, etc. within the previous measure.
      */
-    virtual int SetCurrentScoreDef(FunctorParams *functorParams);
+    virtual int ScoreDefSetCurrent(FunctorParams *functorParams);
 
     /**
      * Optimize the scoreDef for each system.
      * For automatic breaks, looks for staves with only mRests.
      */
     ///@{
-    virtual int OptimizeScoreDef(FunctorParams *) { return FUNCTOR_CONTINUE; }
-    virtual int OptimizeScoreDefEnd(FunctorParams *) { return FUNCTOR_CONTINUE; }
+    virtual int ScoreDefOptimize(FunctorParams *) { return FUNCTOR_CONTINUE; }
+    virtual int ScoreDefOptimizeEnd(FunctorParams *) { return FUNCTOR_CONTINUE; }
     ///@}
 
     /**
@@ -948,7 +997,7 @@ public:
     /**
      * Unset the initial scoreDef of each system and measure
      */
-    virtual int UnsetCurrentScoreDef(FunctorParams *) { return FUNCTOR_CONTINUE; }
+    virtual int ScoreDefUnsetCurrent(FunctorParams *) { return FUNCTOR_CONTINUE; }
 
     /**
      * Set drawing flags for the StaffDef for indicating whether clefs, keysigs, etc. need
@@ -981,7 +1030,7 @@ public:
     /**
      * Prepare group symbol starting and ending staffDefs for drawing
      */
-    virtual int PrepareGroupSymbols(FunctorParams *) { return FUNCTOR_CONTINUE; }
+    virtual int ScoreDefSetGrpSym(FunctorParams *) { return FUNCTOR_CONTINUE; }
 
     /**
      * Associate LayerElement with @facs to the appropriate zone
@@ -1205,8 +1254,6 @@ public:
      */
     virtual int Transpose(FunctorParams *) { return FUNCTOR_CONTINUE; }
 
-    static bool sortByUlx(Object *a, Object *b);
-
 protected:
     //
 private:
@@ -1303,6 +1350,10 @@ private:
      */
     bool m_isExpansion;
 
+    //----------------//
+    // Static members //
+    //----------------//
+
     /**
      * A static counter for uuid generation.
      */
@@ -1486,6 +1537,53 @@ private:
     ClassId m_classId;
 };
 
+//----------------------------------------------------------------------------
+// ObjectFactory
+//----------------------------------------------------------------------------
+
+class ObjectFactory {
+
+public:
+    /**
+     * A static method returning a static object in order to guarantee initialisation
+     */
+    static ObjectFactory *GetInstance();
+
+    /**
+     * Create the object from the MEI element string name by making a lookup in the register
+     */
+    Object *Create(std::string name);
+
+    /**
+     * Add the name / constructor map entry to the static register
+     */
+    void Register(std::string name, ClassId classId, std::function<Object *(void)> function);
+
+    /**
+     * Get the correspondings ClassIds from the vector of MEI element string names
+     */
+    void GetClassIds(const std::vector<std::string> &classStrings, std::vector<ClassId> &classIds);
+
+public:
+    MapOfStrConstructors s_ctorsRegistry;
+    MapOfStrClassIds s_classIdsRegistry;
+};
+
+//----------------------------------------------------------------------------
+// ClassRegistrar
+//----------------------------------------------------------------------------
+
+template <class T> class ClassRegistrar {
+public:
+    /**
+     * The contructor registering the name / constructor map
+     */
+    ClassRegistrar(std::string name, ClassId classId)
+    {
+        ObjectFactory::GetInstance()->Register(name, classId, [](void) -> Object * { return new T(); });
+    }
+};
+
 } // namespace vrv
 
 #endif
