@@ -51,7 +51,7 @@ class OptionGrp;
 // Option defines
 //----------------------------------------------------------------------------
 
-enum option_BREAKS { BREAKS_none = 0, BREAKS_auto, BREAKS_line, BREAKS_encoded };
+enum option_BREAKS { BREAKS_none = 0, BREAKS_auto, BREAKS_line, BREAKS_smart, BREAKS_encoded };
 
 enum option_CONDENSE { CONDENSE_none = 0, CONDENSE_auto, CONDENSE_all, CONDENSE_encoded };
 
@@ -59,7 +59,12 @@ enum option_FOOTER { FOOTER_none = 0, FOOTER_auto, FOOTER_encoded, FOOTER_always
 
 enum option_HEADER { HEADER_none = 0, HEADER_auto, HEADER_encoded };
 
-enum option_MEASURENUMBER { MEASURENUMBER_system = 0, MEASURENUMBER_interval };
+enum option_MULTIRESTSTYLE {
+    MULTIRESTSTYLE_auto = 0,
+    MULTIRESTSTYLE_default,
+    MULTIRESTSTYLE_block,
+    MULTIRESTSTYLE_symbols
+};
 
 enum option_SYSTEMDIVIDER { SYSTEMDIVIDER_none = 0, SYSTEMDIVIDER_auto, SYSTEMDIVIDER_left, SYSTEMDIVIDER_left_right };
 
@@ -73,7 +78,12 @@ enum option_SYSTEMDIVIDER { SYSTEMDIVIDER_none = 0, SYSTEMDIVIDER_auto, SYSTEMDI
 class Option {
 public:
     // constructors and destructors
-    Option() : m_isSet(false) {}
+    Option()
+    {
+        m_isSet = false;
+        m_shortOption = 0;
+        m_isCmdOnly = false;
+    }
     virtual ~Option() {}
     virtual void CopyTo(Option *option);
 
@@ -91,9 +101,22 @@ public:
     std::string GetTitle() const { return m_title; }
     std::string GetDescription() const { return m_description; }
 
-    bool isSet() const { return m_isSet; }
+    bool IsSet() const { return m_isSet; }
+
+    void SetShortOption(char shortOption, bool isCmdOnly);
+    char GetShortOption() const { return m_shortOption; }
+    bool IsCmdOnly() const { return m_isCmdOnly; }
+
+    /**
+     * Return a JSON object for the option
+     */
+    jsonxx::Object ToJson() const;
 
 public:
+    //----------------//
+    // Static members //
+    //----------------//
+
     /**
      * Static maps used my OptionIntMap objects. Set in OptIntMap::Init
      */
@@ -101,7 +124,7 @@ public:
     static std::map<int, std::string> s_condense;
     static std::map<int, std::string> s_footer;
     static std::map<int, std::string> s_header;
-    static std::map<int, std::string> s_measureNumber;
+    static std::map<int, std::string> s_multiRestStyle;
     static std::map<int, std::string> s_systemDivider;
 
 protected:
@@ -111,6 +134,10 @@ protected:
 
 private:
     std::string m_key;
+    /* the character for a short option - not set (0) by default) */
+    char m_shortOption;
+    /* a flag indicating that the option is available only on the command line */
+    bool m_isCmdOnly;
 };
 
 //----------------------------------------------------------------------------
@@ -123,7 +150,7 @@ private:
 class OptionBool : public Option {
 public:
     // constructors and destructors
-    OptionBool() {}
+    OptionBool() { m_defaultValue = false; }
     virtual ~OptionBool() {}
     virtual void CopyTo(Option *option);
     void Init(bool defaultValue);
@@ -157,7 +184,12 @@ private:
 class OptionDbl : public Option {
 public:
     // constructors and destructors
-    OptionDbl() {}
+    OptionDbl()
+    {
+        m_defaultValue = 0.0;
+        m_minValue = 0.0;
+        m_maxValue = 0.0;
+    }
     virtual ~OptionDbl() {}
     virtual void CopyTo(Option *option);
     void Init(double defaultValue, double minValue, double maxValue);
@@ -194,7 +226,12 @@ private:
 class OptionInt : public Option {
 public:
     // constructors and destructors
-    OptionInt() {}
+    OptionInt()
+    {
+        m_defaultValue = 0;
+        m_minValue = 0;
+        m_maxValue = 0;
+    }
     virtual ~OptionInt() {}
     virtual void CopyTo(Option *option);
     void Init(int defaultValue, int minValue, int maxValue, bool definitionFactor = false);
@@ -398,7 +435,7 @@ private:
  */
 
 class OptionJson : public Option {
-    using JsonPath = std::vector<std::reference_wrapper<jsonxx::Value> >;
+    using JsonPath = std::vector<std::reference_wrapper<jsonxx::Value>>;
 
 public:
     //
@@ -475,6 +512,10 @@ public:
 
     std::vector<OptionGrp *> *GetGrps() { return &m_grps; }
 
+    jsonxx::Object GetBaseOptGrp();
+
+    const std::vector<Option *> *GetBaseOptions();
+
     // post processing of parameters
     void Sync();
 
@@ -485,6 +526,21 @@ public:
     /**
      * Comments in implementation file options.cpp
      */
+    OptionGrp m_baseOptions;
+
+    // These options are only given for documentation - except for m_scale
+    // They are ordered by short option alphabetical order
+    OptionBool m_standardOutput;
+    OptionBool m_help;
+    OptionBool m_allPpages;
+    OptionString m_inputFrom;
+    OptionString m_outfile;
+    OptionInt m_page;
+    OptionString m_resourcePath;
+    OptionInt m_scale;
+    OptionString m_outputTo;
+    OptionBool m_version;
+    OptionInt m_xmlIdSeed;
 
     /**
      * General
@@ -494,10 +550,12 @@ public:
     OptionBool m_adjustPageHeight;
     OptionBool m_adjustPageWidth;
     OptionIntMap m_breaks;
+    OptionDbl m_breaksSmartSb;
     OptionIntMap m_condense;
     OptionBool m_condenseFirstPage;
     OptionBool m_condenseTempoPages;
     OptionBool m_evenNoteSpacing;
+    OptionString m_expand;
     OptionBool m_humType;
     OptionBool m_justifyVertically;
     OptionBool m_landscape;
@@ -509,6 +567,7 @@ public:
     OptionIntMap m_header;
     OptionBool m_noJustification;
     OptionBool m_openControlEvents;
+    OptionBool m_outputFormatRaw;
     OptionInt m_outputIndent;
     OptionBool m_outputIndentTab;
     OptionBool m_outputSmuflXmlEntities;
@@ -518,11 +577,14 @@ public:
     OptionInt m_pageMarginRight;
     OptionInt m_pageMarginTop;
     OptionInt m_pageWidth;
-    OptionString m_expand;
+    OptionBool m_preserveAnalyticalMarkup;
+    OptionBool m_removeIds;
     OptionBool m_shrinkToFit;
     OptionBool m_svgBoundingBoxes;
     OptionBool m_svgViewBox;
     OptionBool m_svgHtml5;
+    OptionBool m_svgFormatRaw;
+    OptionBool m_svgRemoveXlink;
     OptionInt m_unit;
     OptionBool m_useFacsimile;
     OptionBool m_usePgFooterForAll;
@@ -539,6 +601,8 @@ public:
     OptionInt m_beamMaxSlope;
     OptionInt m_beamMinSlope;
     OptionDbl m_bracketThickness;
+    OptionDbl m_dynamDist;
+    OptionDbl m_clefChangeFactor;
     OptionJson m_engravingDefaults;
     OptionString m_font;
     OptionDbl m_graceFactor;
@@ -546,6 +610,7 @@ public:
     OptionBool m_graceRightAlign;
     OptionDbl m_hairpinSize;
     OptionDbl m_hairpinThickness;
+    OptionDbl m_harmDist;
     OptionDbl m_justificationBraceGroup;
     OptionDbl m_justificationBracketGroup;
     OptionDbl m_justificationStaff;
@@ -559,7 +624,10 @@ public:
     OptionDbl m_lyricTopMinMargin;
     OptionDbl m_lyricWordSpace;
     OptionInt m_measureMinWidth;
-    OptionIntMap m_measureNumber;
+    OptionInt m_mnumInterval;
+    OptionIntMap m_multiRestStyle;
+    OptionBool m_octaveAlternativeSymbols;
+    OptionDbl m_octaveLineThickness;
     OptionDbl m_repeatBarLineDotSeparation;
     OptionDbl m_repeatEndingLineThickness;
     OptionInt m_slurControlPoints;
@@ -587,6 +655,7 @@ public:
     OptionDbl m_tieEndpointThickness;
     OptionDbl m_tieMidpointThickness;
     OptionDbl m_tupletBracketThickness;
+    OptionBool m_tupletNumHead;
 
     /**
      * Selectors
@@ -610,6 +679,7 @@ public:
     OptionDbl m_defaultRightMargin;
     OptionDbl m_defaultTopMargin;
     //
+    OptionDbl m_bottomMarginArtic;
     OptionDbl m_bottomMarginHarm;
     OptionDbl m_bottomMarginPgHead;
     //
@@ -629,6 +699,7 @@ public:
     OptionDbl m_leftMarginNote;
     OptionDbl m_leftMarginRest;
     OptionDbl m_leftMarginRightBarLine;
+    OptionDbl m_leftMarginTabDurSym;
     //
     OptionDbl m_rightMarginAccid;
     OptionDbl m_rightMarginBarLine;
@@ -646,7 +717,9 @@ public:
     OptionDbl m_rightMarginNote;
     OptionDbl m_rightMarginRest;
     OptionDbl m_rightMarginRightBarLine;
+    OptionDbl m_rightMarginTabDurSym;
     //
+    OptionDbl m_topMarginArtic;
     OptionDbl m_topMarginHarm;
 
     /**
@@ -654,10 +727,6 @@ public:
      */
     OptionGrp m_deprecated;
 
-    OptionBool m_condenseEncoded;
-    OptionDbl m_slurThickness;
-    OptionDbl m_tieThickness;
-
 private:
     /** The array of style parameters */
     MapOfStrOptions m_items;
