@@ -9,8 +9,11 @@
 
 //----------------------------------------------------------------------------
 
+#include <algorithm>
 #include <assert.h>
+#include <functional>
 #include <iostream>
+#include <numeric>
 
 //----------------------------------------------------------------------------
 
@@ -31,10 +34,34 @@
 
 namespace vrv {
 
+// Helper template function to calculate optiomal dot locations based on note locations in the chord. Takes iterators
+// for the begin/end of the range; reverse iterators should be passed if reverse order is specified
+template <typename Iterator> std::set<int> CalculateDotLocations(Iterator begin, Iterator end, bool isReverseOrder)
+{
+    // location adjustment that should be applied when looking for optimal position
+    std::vector<int> locAdjust{ 1, 2, -3 };
+    if (isReverseOrder) std::transform(locAdjust.begin(), locAdjust.end(), locAdjust.begin(), std::negate<int>());
+    std::set<int> dotLocations;
+    for (auto iter = begin; iter != end; ++iter) {
+        bool result = false;
+        if (*iter % 2 != 0) std::tie(std::ignore, result) = dotLocations.insert(*iter);
+        if (!result) {
+            for (auto adjust : locAdjust) {
+                if ((*iter + adjust) % 2 == 0) continue;
+                std::tie(std::ignore, result) = dotLocations.insert(*iter + adjust);
+                if (result) break;
+            }
+        }
+    }
+    return dotLocations;
+}
+
 //----------------------------------------------------------------------------
 // Chord
 //----------------------------------------------------------------------------
 
+static ClassRegistrar<Chord> s_factory("chord", CHORD);
+
 Chord::Chord()
     : LayerElement("chord-")
     , ObjectListInterface()
@@ -290,7 +317,7 @@ int Chord::GetXMax()
     return x;
 }
 
-void Chord::GetCrossStaffExtremes(Staff *&staffAbove, Staff *&staffBelow)
+void Chord::GetCrossStaffExtremes(Staff *&staffAbove, Staff *&staffBelow, Layer **layerAbove, Layer **layerBelow)
 {
     staffAbove = NULL;
     staffBelow = NULL;
@@ -303,6 +330,7 @@ void Chord::GetCrossStaffExtremes(Staff *&staffAbove, Staff *&staffBelow)
     assert(bottomNote);
     if (bottomNote->m_crossStaff && bottomNote->m_crossLayer) {
         staffBelow = bottomNote->m_crossStaff;
+        if (layerBelow) (*layerBelow) = bottomNote->m_crossLayer;
     }
 
     // The last note is the top
@@ -310,31 +338,7 @@ void Chord::GetCrossStaffExtremes(Staff *&staffAbove, Staff *&staffBelow)
     assert(topNote);
     if (topNote->m_crossStaff && topNote->m_crossLayer) {
         staffAbove = topNote->m_crossStaff;
-    }
-}
-
-void Chord::GetCrossStaffOverflows(LayerElement *element, StaffAlignment *alignment, bool &skipAbove, bool &skipBelow)
-{
-    assert(element);
-    assert(alignment);
-
-    // Only flags and stems need to be skipped
-    if (!element->Is({ FLAG, STEM })) return;
-
-    // Nothing to do if there is not cross-staff
-    if (!this->HasCrossStaff()) return;
-
-    Staff *staff = alignment->GetStaff();
-    assert(staff);
-
-    Staff *staffAbove = NULL;
-    Staff *staffBelow = NULL;
-    this->GetCrossStaffExtremes(staffAbove, staffBelow);
-    if (staffAbove && (staffAbove != staff)) {
-        skipAbove = true;
-    }
-    if (staffBelow && (staffBelow != staff)) {
-        skipBelow = true;
+        if (layerAbove) (*layerAbove) = topNote->m_crossLayer;
     }
 }
 
@@ -364,19 +368,22 @@ Point Chord::GetStemDownNW(Doc *doc, int staffSize, bool isCueSize)
     return topNote->GetStemDownNW(doc, staffSize, isCueSize);
 }
 
-int Chord::CalcStemLenInThirdUnits(Staff *staff)
+int Chord::CalcStemLenInThirdUnits(Staff *staff, data_STEMDIRECTION stemDir)
 {
     assert(staff);
 
-    if (this->GetDrawingStemDir() == STEMDIRECTION_up) {
+    if (stemDir == STEMDIRECTION_up) {
         Note *topNote = this->GetTopNote();
         assert(topNote);
-        return topNote->CalcStemLenInThirdUnits(staff);
+        return topNote->CalcStemLenInThirdUnits(staff, stemDir);
     }
-    else {
+    else if (stemDir == STEMDIRECTION_down) {
         Note *bottomNote = this->GetBottomNote();
         assert(bottomNote);
-        return bottomNote->CalcStemLenInThirdUnits(staff);
+        return bottomNote->CalcStemLenInThirdUnits(staff, stemDir);
+    }
+    else {
+        return 0;
     }
 }
 
@@ -386,7 +393,7 @@ bool Chord::IsVisible()
         return this->GetVisible() == BOOLEAN_true;
     }
 
-    // if the chord doens't have it, see if all the children are invisible
+    // if the chord doesn't have it, see if all the children are invisible
     const ArrayOfObjects *notes = this->GetList(this);
     assert(notes);
 
@@ -447,8 +454,8 @@ void Chord::AdjustOverlappingLayers(Doc *doc, const std::vector<LayerElement *>
     for (auto iter : *notes) {
         Note *note = vrv_cast<Note *>(iter);
         assert(note);
-        auto [overlap, isInUnison]
-            = note->CalcNoteHorizontalOverlap(doc, otherElements, true, isLowerPosition, expectedElementsInUnison > 0);
+        auto [overlap, isInUnison] = note->CalcElementHorizontalOverlap(
+            doc, otherElements, true, isLowerPosition, expectedElementsInUnison > 0);
         if (((margin >= 0) && (overlap > margin)) || ((margin <= 0) && (overlap < margin))) {
             margin = overlap;
         }
@@ -519,6 +526,64 @@ int Chord::ConvertMarkupAnalyticalEnd(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
+int Chord::CalcArtic(FunctorParams *functorParams)
+{
+    CalcArticParams *params = vrv_params_cast<CalcArticParams *>(functorParams);
+    assert(params);
+
+    params->m_parent = this;
+    params->m_stemDir = this->GetDrawingStemDir();
+
+    Staff *staff = vrv_cast<Staff *>(this->GetFirstAncestor(STAFF));
+    assert(staff);
+    Layer *layer = vrv_cast<Layer *>(this->GetFirstAncestor(LAYER));
+    assert(layer);
+
+    params->m_staffAbove = staff;
+    params->m_staffBelow = staff;
+    params->m_layerAbove = layer;
+    params->m_layerBelow = layer;
+    params->m_crossStaffAbove = false;
+    params->m_crossStaffBelow = false;
+
+    if (this->m_crossStaff) {
+        params->m_staffAbove = this->m_crossStaff;
+        params->m_staffBelow = this->m_crossStaff;
+        params->m_layerAbove = this->m_crossLayer;
+        params->m_layerBelow = this->m_crossLayer;
+        params->m_crossStaffAbove = true;
+        params->m_crossStaffBelow = true;
+    }
+    else {
+        this->GetCrossStaffExtremes(
+            params->m_staffAbove, params->m_staffBelow, &params->m_layerAbove, &params->m_layerBelow);
+        if (params->m_staffAbove) {
+            params->m_crossStaffAbove = true;
+            params->m_staffBelow = staff;
+            params->m_layerBelow = layer;
+        }
+        else if (params->m_staffBelow) {
+            params->m_crossStaffBelow = true;
+            params->m_staffAbove = staff;
+            params->m_layerAbove = layer;
+        }
+    }
+
+    return FUNCTOR_CONTINUE;
+}
+
+int Chord::AdjustArtic(FunctorParams *functorParams)
+{
+    AdjustArticParams *params = vrv_params_cast<AdjustArticParams *>(functorParams);
+    assert(params);
+
+    params->m_parent = this;
+    params->m_articAbove.clear();
+    params->m_articBelow.clear();
+
+    return FUNCTOR_CONTINUE;
+}
+
 int Chord::CalcStem(FunctorParams *functorParams)
 {
     CalcStemParams *params = vrv_params_cast<CalcStemParams *>(functorParams);
@@ -527,7 +592,7 @@ int Chord::CalcStem(FunctorParams *functorParams)
     // Set them to NULL in any case
     params->m_interface = NULL;
 
-    // Stems have been calculated previously in Beam or FTrem - siblings becasue flags do not need to
+    // Stems have been calculated previously in beam or fTrem - siblings because flags do not need to
     // be processed either
     if (this->IsInBeam() || this->IsInFTrem()) {
         return FUNCTOR_SIBLINGS;
@@ -592,6 +657,50 @@ int Chord::CalcStem(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
+void Chord::CaltOptimalDots(Dots *dots, Staff *staff, const std::set<int> &noteLocations)
+{
+    // calculate optimal dot locations both in normal and reverse orders
+    std::set<int> forwardLocations = CalculateDotLocations(noteLocations.begin(), noteLocations.end(), false);
+    std::set<int> backwardLocations = CalculateDotLocations(noteLocations.rbegin(), noteLocations.rend(), true);
+    //
+    std::vector<int> firstElem, secondElem;
+    const std::set<int> *firstRef, *secondRef;
+    Staff *currentStaff = vrv_cast<Staff *>(this->GetFirstAncestor(STAFF));
+    const bool isUpwardDirection
+        = (GetDrawingStemDir() == STEMDIRECTION_up) || (currentStaff->GetChildCount(LAYER) == 1);
+    // On upper layer normal order positioning is prioritized, hence assign positions in the same order as they were
+    // calculated. This way, if differences in positioning are the same for both normal/reverse orders, normal is going
+    // to be selected
+    if (isUpwardDirection) {
+        firstElem.assign(forwardLocations.begin(), forwardLocations.end());
+        secondElem.assign(backwardLocations.begin(), backwardLocations.end());
+        firstRef = &forwardLocations;
+        secondRef = &backwardLocations;
+    }
+    // ... and vice versa for the bottom layer, where reverse ordering is in priority
+    else {
+        firstElem.assign(backwardLocations.begin(), backwardLocations.end());
+        secondElem.assign(forwardLocations.begin(), forwardLocations.end());
+        firstRef = &backwardLocations;
+        secondRef = &forwardLocations;
+    }
+
+    // substract note and dot positions and calculate difference between all locations. This way, better positioning can
+    // be determined by taking order, where difference between dot and note positions is the smallest
+    std::transform(firstElem.begin(), firstElem.end(), noteLocations.begin(), firstElem.begin(), std::minus<int>());
+    std::transform(secondElem.begin(), secondElem.end(), noteLocations.begin(), secondElem.begin(), std::minus<int>());
+    // apply std::abs to elements in the vectors and then calculate sum of the elements. Can be achieved with
+    // std::transform_reduce, but gcc needs to support it
+    std::transform(firstElem.begin(), firstElem.end(), firstElem.begin(), static_cast<float (*)(float)>(&std::abs));
+    std::transform(secondElem.begin(), secondElem.end(), secondElem.begin(), static_cast<float (*)(float)>(&std::abs));
+    const int firstDiff = std::accumulate(firstElem.begin(), firstElem.end(), 0);
+    const int secondDiff = std::accumulate(secondElem.begin(), secondElem.end(), 0);
+    std::set<int> dotLocations = (secondDiff < firstDiff) ? *secondRef : *firstRef;
+
+    std::list<int> *dotLocs = dots->GetDotLocsForStaff(staff);
+    dotLocs->insert(dotLocs->end(), dotLocations.begin(), dotLocations.end());
+}
+
 int Chord::CalcDots(FunctorParams *functorParams)
 {
     CalcDotsParams *params = vrv_params_cast<CalcDotsParams *>(functorParams);
@@ -625,65 +734,25 @@ int Chord::CalcDots(FunctorParams *functorParams)
     assert(this->GetTopNote());
     assert(this->GetBottomNote());
 
+    // Get note locations first
+    std::map<Staff *, std::set<int>> noteLocations;
     for (rit = notes->rbegin(); rit != notes->rend(); ++rit) {
         Note *note = vrv_cast<Note *>(*rit);
         assert(note);
-
         if (note->GetDots() == 0) {
             continue;
         }
 
         Layer *layer = NULL;
         Staff *staff = note->GetCrossStaff(layer);
-
-        std::list<int> *dotLocs = dots->GetDotLocsForStaff(staff);
-        int loc = note->GetDrawingLoc();
-
-        // if it's on a staff line to start with, we need to compensate here and add a full unit like DrawDots would
-        if ((loc % 2) == 0) {
-            // defaults to the space above the staffline first
-            // if that position is not on the list already, we're good to go
-            if (std::find(dotLocs->begin(), dotLocs->end(), loc + 1) == dotLocs->end()) {
-                loc += 1;
-            }
-            // if it is on the list, we should try the spot a doubleUnit below
-            else if (std::find(dotLocs->begin(), dotLocs->end(), loc - 1) == dotLocs->end()) {
-                loc -= 1;
-            }
-            // otherwise, any other space looks weird so let's not draw it
-            else {
-                continue;
-            }
-        }
-        // similar if it's not on a staff line
-        else {
-            // see if the optimal place exists already
-            if (std::find(dotLocs->begin(), dotLocs->end(), loc) == dotLocs->end()) {
-            }
-            // if it does, then look up a space first
-            else if (std::find(dotLocs->begin(), dotLocs->end(), loc + 2) == dotLocs->end()) {
-                loc += 2;
-            }
-            // then look down a space
-            else if (std::find(dotLocs->begin(), dotLocs->end(), loc - 2) == dotLocs->end()) {
-                loc -= 2;
-            }
-            // otherwise let's not draw it
-            else {
-                continue;
-            }
+        if (noteLocations.end() == noteLocations.find(staff)) {
+            noteLocations[staff] = {};
         }
+        noteLocations[staff].insert(note->GetDrawingLoc());
+    }
 
-        // finally, make sure it's not outside the acceptable extremes of the chord.
-        // however, this does take into account cross-staff chords because it looks only at the top and bottom notes.
-        // when it would be necessary to look at top and bottom for each staff
-        if (!this->HasCrossStaff()) {
-            if (loc > this->GetTopNote()->GetDrawingLoc() + 1) continue;
-            if (loc < this->GetBottomNote()->GetDrawingLoc() - 1) continue;
-        }
-
-        // if it's not, add it to the dots list and go back to DrawChord
-        dotLocs->push_back(loc);
+    for (const auto &loc : noteLocations) {
+        CaltOptimalDots(dots, loc.first, loc.second);
     }
 
     return FUNCTOR_CONTINUE;
@@ -702,7 +771,7 @@ int Chord::PrepareLayerElementParts(FunctorParams *functorParams)
     currentStem->AttGraced::operator=(*this);
     currentStem->AttStems::operator=(*this);
     currentStem->AttStemsCmn::operator=(*this);
-    if (this->GetActualDur() < DUR_2) {
+    if (this->GetActualDur() < DUR_2 || (this->GetStemVisible() == BOOLEAN_false)) {
         currentStem->IsVirtual(true);
     }
 
