@@ -43,6 +43,7 @@ void Dots::Reset()
     LayerElement::Reset();
     ResetAugmentDots();
 
+    m_isAdjusted = false;
     m_dotLocsByStaff.clear();
 }
 
@@ -70,7 +71,7 @@ void Flag::Reset()
     m_drawingNbFlags = 0;
 }
 
-wchar_t Flag::GetSmuflCode(data_STEMDIRECTION stemDir)
+wchar_t Flag::GetFlagGlyph(data_STEMDIRECTION stemDir)
 {
     if (stemDir == STEMDIRECTION_up) {
         switch (m_drawingNbFlags) {
@@ -102,7 +103,7 @@ wchar_t Flag::GetSmuflCode(data_STEMDIRECTION stemDir)
 
 Point Flag::GetStemUpSE(Doc *doc, int staffSize, bool graceSize, wchar_t &code)
 {
-    code = this->GetSmuflCode(STEMDIRECTION_up);
+    code = this->GetFlagGlyph(STEMDIRECTION_up);
 
     int h = doc->GetGlyphHeight(code, staffSize, graceSize);
     return Point(0, h + doc->GetGlyphDescender(code, staffSize, graceSize));
@@ -110,7 +111,7 @@ Point Flag::GetStemUpSE(Doc *doc, int staffSize, bool graceSize, wchar_t &code)
 
 Point Flag::GetStemDownNW(Doc *doc, int staffSize, bool graceSize, wchar_t &code)
 {
-    code = this->GetSmuflCode(STEMDIRECTION_down);
+    code = this->GetFlagGlyph(STEMDIRECTION_down);
 
     return Point(0, doc->GetGlyphDescender(code, staffSize, graceSize));
 }
@@ -295,38 +296,30 @@ bool Stem::IsSupportedChild(Object *child)
     return true;
 }
 
-void Stem::AdjustOverlappingLayers(Doc *doc, const std::vector<LayerElement *> &otherElements, bool &isUnison)
+int Stem::CompareToElementPosition(Doc *doc, LayerElement *otherElement, int margin)
 {
-    if (isUnison) {
-        isUnison = false;
-        return;
-    }
     Staff *staff = vrv_cast<Staff *>(GetFirstAncestor(STAFF));
     assert(staff);
-    // check if there is an overlap on the left or on the right and displace stem's parent correspondigly
-    for (auto element : otherElements) {
-        int right = HorizontalLeftOverlap(element, doc, 0, 0);
-        int left = HorizontalRightOverlap(element, doc, 0, 0);
-        if (!right || !left) continue;
 
-        LayerElement *parent = vrv_cast<LayerElement *>(GetParent());
-        assert(parent);
-        int horizontalMargin = 2 * doc->GetDrawingStemWidth(staff->m_drawingStaffSize);
-        Flag *currentFlag = NULL;
-        currentFlag = vrv_cast<Flag *>(FindDescendantByType(FLAG, 1));
-        if (currentFlag) {
-            wchar_t flagGlyph = currentFlag->GetSmuflCode(STEMDIRECTION_down);
-            const int flagWidth = doc->GetGlyphWidth(flagGlyph, staff->m_drawingStaffSize, GetDrawingCueSize());
-            horizontalMargin += flagWidth;
-        }
+    // check if there is an overlap on the left or on the right and displace stem's parent correspondingly
+    const int right = HorizontalLeftOverlap(otherElement, doc, margin, 0);
+    const int left = HorizontalRightOverlap(otherElement, doc, margin, 0);
+    if (!right || !left) return 0;
 
-        if (right < left) {
-            parent->SetDrawingXRel(parent->GetDrawingXRel() + right + horizontalMargin);
-        }
-        else {
-            parent->SetDrawingXRel(parent->GetDrawingXRel() - horizontalMargin - left);
-        }
-        return;
+    int horizontalMargin = 2 * doc->GetDrawingStemWidth(staff->m_drawingStaffSize);
+    Flag *currentFlag = NULL;
+    currentFlag = vrv_cast<Flag *>(FindDescendantByType(FLAG, 1));
+    if (currentFlag && currentFlag->m_drawingNbFlags) {
+        wchar_t flagGlyph = currentFlag->GetFlagGlyph(STEMDIRECTION_down);
+        const int flagWidth = doc->GetGlyphWidth(flagGlyph, staff->m_drawingStaffSize, GetDrawingCueSize());
+        horizontalMargin += flagWidth;
+    }
+
+    if (right < left) {
+        return right + horizontalMargin;
+    }
+    else {
+        return (-horizontalMargin - left);
     }
 }
 
@@ -337,7 +330,8 @@ void Stem::AdjustFlagPlacement(Doc *doc, Flag *flag, int staffSize, int vertical
 
     const data_STEMDIRECTION stemDirection = GetDrawingStemDir();
     // For overlapping purposes we don't care for flags shorter than 16th since they grow in opposite direction
-    const wchar_t flagGlyph = (duration >= DURATION_16) ? SMUFL_E242_flag16thUp : flag->GetSmuflCode(stemDirection);
+    wchar_t flagGlyph = SMUFL_E242_flag16thUp;
+    if (duration < DURATION_16) flagGlyph = flag->GetFlagGlyph(stemDirection);
     const int glyphHeight = doc->GetGlyphHeight(flagGlyph, staffSize, GetDrawingCueSize());
     const int relevantGlyphHeight = (stemDirection == STEMDIRECTION_up) ? glyphHeight / 2 : glyphHeight;
 
@@ -396,6 +390,7 @@ int Dots::ResetDrawing(FunctorParams *functorParams)
     LayerElement::ResetDrawing(functorParams);
 
     m_dotLocsByStaff.clear();
+    m_isAdjusted = false;
 
     return FUNCTOR_CONTINUE;
 }
@@ -470,6 +465,14 @@ int Stem::CalcStem(FunctorParams *functorParams)
     int stemShift = params->m_doc->GetDrawingStemWidth(staffSize) / 2;
     bool drawingCueSize = this->GetDrawingCueSize();
 
+    // For notes longer than half notes the stem is always 0
+    if (params->m_dur < DUR_2) {
+        this->SetDrawingXRel(0);
+        this->SetDrawingYRel(0);
+        this->SetDrawingStemLen(0);
+        return FUNCTOR_CONTINUE;
+    }
+
     /************ Set the position, the length and adjust to the note head ************/
 
     int baseStem = 0;
@@ -479,7 +482,8 @@ int Stem::CalcStem(FunctorParams *functorParams)
     }
     else {
         int thirdUnit = params->m_doc->GetDrawingUnit(staffSize) / 3;
-        baseStem = -(params->m_interface->CalcStemLenInThirdUnits(params->m_staff) * thirdUnit);
+        const data_STEMDIRECTION stemDir = params->m_interface->GetDrawingStemDir();
+        baseStem = -(params->m_interface->CalcStemLenInThirdUnits(params->m_staff, stemDir) * thirdUnit);
         if (drawingCueSize) baseStem = params->m_doc->GetCueSize(baseStem);
     }
     // Even if a stem length is given we add the length of the chord content (however only if not 0)
@@ -522,7 +526,9 @@ int Stem::CalcStem(FunctorParams *functorParams)
         flag = vrv_cast<Flag *>(this->GetFirst(FLAG));
         assert(flag);
         flag->m_drawingNbFlags = params->m_dur - DUR_4;
-        if (!this->HasStemLen() && this->HasStemMod()) slashFactor += (params->m_dur > DUR_8) ? 2 : 1;
+        if (!this->HasStemLen() && !this->IsGraceNote() && this->HasStemMod()) {
+            slashFactor += (params->m_dur > DUR_8) ? 2 : 1;
+        }
     }
 
     // Adjust basic stem length to number of slashes
