@@ -88,6 +88,9 @@ bool PAEOutput::Export(std::string &output)
 
 bool PAEOutput::WriteObject(Object *object)
 {
+    if (object->Is(MDIV)) {
+        WriteMdiv(dynamic_cast<Mdiv *>(object));
+    }
     if (object->Is(SCOREDEF)) {
         WriteScoreDef(dynamic_cast<ScoreDef *>(object));
     }
@@ -181,6 +184,13 @@ bool PAEOutput::WriteObjectEnd(Object *object)
     return true;
 }
 
+void PAEOutput::WriteMdiv(Mdiv *mdiv)
+{
+    assert(mdiv);
+
+    m_streamStringOutput << "@data:";
+}
+
 void PAEOutput::WriteScoreDef(ScoreDef *scoreDef) {}
 
 void PAEOutput::WriteStaffDef(StaffDef *staffDef)
@@ -238,7 +248,6 @@ void PAEOutput::WriteLayer(Layer *layer)
 
     if (m_layerN == -1) {
         m_layerN = layer->GetN();
-        m_streamStringOutput << "@data:";
         m_skip = false;
         return;
     }
@@ -256,7 +265,7 @@ void PAEOutput::WriteBarLine(BarLine *barLine)
 
     if (m_skip) return;
 
-    // We should look more precisely at its apperance...
+    // We should look more precisely at its appearance ...
     m_streamStringOutput << "/";
 }
 
@@ -715,6 +724,9 @@ void PAEInput::parsePlainAndEasy(std::istream &infile)
         else if (strcmp(data_key, "data") == 0) {
             strcpy(incipit, data_value);
         }
+        else {
+            LogWarning("Unknown row '%s' in incipit data", data_line);
+        }
     }
 
     if (strlen(c_clef)) {
@@ -1016,8 +1028,7 @@ void PAEInput::parsePlainAndEasy(std::istream &infile)
     m_doc->m_mdivScoreDef.AddChild(staffGrp);
 
     if (m_tie != NULL) {
-        delete m_tie;
-        m_tie = NULL;
+        LogWarning("Open tie will not render because tstamp2 is missing");
     }
 
     m_doc->ConvertToPageBasedDoc();
@@ -1526,7 +1537,7 @@ int PAEInput::getWholeRest(const char *incipit, int *wholerest, int index)
 /**********************************
  *
  * getBarLine -- read the barLine.
- * Translation from PAE to verovio representaion:
+ * Translation from PAE to verovio representation:
  *
  BARRENDITION_single     /
  BARRENDITION_end        does not exist
@@ -1603,6 +1614,12 @@ int PAEInput::getAbbreviation(const char *incipit, pae::Measure *measure, int in
             i++;
             for (int j = measure->abbreviation_offset; j < abbreviation_stop; ++j) {
                 measure->notes.push_back(measure->notes.at(j));
+                // With abbreviation, repeat clefs but do not copy keySig, meterSig and mensur
+                if (measure->notes.back().clef)
+                    measure->notes.back().clef = vrv_cast<Clef *>(measure->notes.back().clef->Clone());
+                measure->notes.back().meter = NULL;
+                measure->notes.back().key = NULL;
+                measure->notes.back().mensur = NULL;
             }
         }
         measure->abbreviation_offset = -1;
@@ -1947,20 +1964,24 @@ void PAEInput::parseNote(pae::Note *note)
     // Does this note have a clef change? push it before everything else
     if (note->clef) {
         addLayerElement(note->clef);
+        note->clef = NULL;
     }
 
     // Same thing for time changes
     // You can find this sometimes
     if (note->meter) {
         addLayerElement(note->meter);
+        note->meter = NULL;
     }
     if (note->mensur) {
         addLayerElement(note->mensur);
+        note->mensur = NULL;
     }
 
     // Handle key change. Evil if done in a beam
     if (note->key) {
         addLayerElement(note->key);
+        note->key = NULL;
     }
 
     // Acciaccaturas are similar but do not get beamed (do they)
@@ -1999,7 +2020,7 @@ void PAEInput::parseNote(pae::Note *note)
     }
 
     // note in a chord
-    if (note->chord) {
+    if (note->chord && element->Is(NOTE)) {
         Note *mnote = dynamic_cast<Note *>(element);
         assert(mnote);
         // first note?
@@ -2016,11 +2037,11 @@ void PAEInput::parseNote(pae::Note *note)
 
     // Add the note to the current container
     addLayerElement(element);
+    element = NULL;
 
     // Add mensural dot
     if (m_is_mensural && note->dots > 0) {
-        Dot *dot = new Dot();
-        addLayerElement(dot);
+        addLayerElement(new Dot());
     }
 
     // the last note counts always '1'
@@ -2037,9 +2058,10 @@ void PAEInput::parseNote(pae::Note *note)
     // last note of a chord
     if (!note->chord && m_is_in_chord) {
         Note *mnote = dynamic_cast<Note *>(element);
-        assert(mnote);
-        mnote->ResetAugmentDots();
-        mnote->ResetDurationLogical();
+        if (mnote) {
+            mnote->ResetAugmentDots();
+            mnote->ResetDurationLogical();
+        }
         popContainer();
         m_is_in_chord = false;
     }
@@ -2066,7 +2088,13 @@ void PAEInput::popContainer()
 void PAEInput::addLayerElement(LayerElement *element)
 {
     if (m_nested_objects.size() > 0) {
-        m_nested_objects.back()->AddChild(element);
+        if (!m_nested_objects.back()->IsSupportedChild(element)) {
+            delete element;
+            element = NULL;
+        }
+        else {
+            m_nested_objects.back()->AddChild(element);
+        }
     }
     else {
         m_layer->AddChild(element);
@@ -2122,6 +2150,7 @@ void PAEInput::getAtRecordKeyValue(char *key, char *value, const char *input)
     // start storing the key value:
     while ((index < length) && (input[index] != SEPARATOR)) {
         if (isspace(input[index])) {
+            index++;
             continue;
         }
         ch = input[index];
