@@ -29,6 +29,7 @@
 #include "smufl.h"
 #include "staff.h"
 #include "syl.h"
+#include "tabgrp.h"
 #include "tie.h"
 #include "transposition.h"
 #include "verse.h"
@@ -44,6 +45,8 @@ namespace vrv {
 // Note
 //----------------------------------------------------------------------------
 
+static ClassRegistrar<Note> s_factory("note", NOTE);
+
 Note::Note()
     : LayerElement("note-")
     , StemmedDrawingInterface()
@@ -56,6 +59,7 @@ Note::Note()
     , AttExtSym()
     , AttGraced()
     , AttMidiVelocity()
+    , AttNoteGesTab()
     , AttNoteHeads()
     , AttNoteVisMensural()
     , AttStems()
@@ -71,6 +75,7 @@ Note::Note()
     RegisterAttClass(ATT_CUE);
     RegisterAttClass(ATT_EXTSYM);
     RegisterAttClass(ATT_GRACED);
+    RegisterAttClass(ATT_NOTEGESTAB);
     RegisterAttClass(ATT_NOTEHEADS);
     RegisterAttClass(ATT_NOTEVISMENSURAL);
     RegisterAttClass(ATT_MIDIVELOCITY);
@@ -96,6 +101,7 @@ void Note::Reset()
     ResetCue();
     ResetExtSym();
     ResetGraced();
+    ResetNoteGesTab();
     ResetNoteHeads();
     ResetNoteVisMensural();
     ResetMidiVelocity();
@@ -121,7 +127,7 @@ void Note::Reset()
 
 bool Note::IsSupportedChild(Object *child)
 {
-    // additional verification for accid and artic - this will no be raised with editorial markup, though
+    // additional verification for accid and artic - this will not be raised with editorial markup, though
     if (child->Is(ACCID)) {
         IsAttributeComparison isAttributeComparison(ACCID);
         if (this->FindDescendantByComparison(&isAttributeComparison))
@@ -233,6 +239,79 @@ bool Note::IsClusterExtreme() const
         return false;
 }
 
+TabGrp *Note::IsTabGrpNote() const
+{
+    return dynamic_cast<TabGrp *>(this->GetFirstAncestor(TABGRP, MAX_TABGRP_DEPTH));
+}
+
+std::wstring Note::GetTabFretString(data_NOTATIONTYPE notationType)
+{
+    if (notationType == NOTATIONTYPE_tab_lute_italian) {
+        std::wstring fretStr;
+        int fret = this->GetTabFret();
+        // Maximum allowed would be 19 (always bindly addind 1 as first figure)
+        if (fret > 9) fretStr.push_back(SMUFL_EBE1_luteItalianFret1);
+        switch (fret % 10) {
+            case 0: fretStr.push_back(SMUFL_EBE0_luteItalianFret0); break;
+            case 1: fretStr.push_back(SMUFL_EBE1_luteItalianFret1); break;
+            case 2: fretStr.push_back(SMUFL_EBE2_luteItalianFret2); break;
+            case 3: fretStr.push_back(SMUFL_EBE3_luteItalianFret3); break;
+            case 4: fretStr.push_back(SMUFL_EBE4_luteItalianFret4); break;
+            case 5: fretStr.push_back(SMUFL_EBE5_luteItalianFret5); break;
+            case 6: fretStr.push_back(SMUFL_EBE6_luteItalianFret6); break;
+            case 7: fretStr.push_back(SMUFL_EBE7_luteItalianFret7); break;
+            case 8: fretStr.push_back(SMUFL_EBE8_luteItalianFret8); break;
+            case 9: fretStr.push_back(SMUFL_EBE9_luteItalianFret9); break;
+            default: break;
+        }
+        return fretStr;
+    }
+    else if (notationType == NOTATIONTYPE_tab_lute_french) {
+        std::wstring fretStr;
+        const int fret = GetTabFret();
+        const int course = GetTabCourse();
+        if (course >= 11) {
+            // french tab uses number 4 ... for courses 11 ..., always open fret a.
+            // TODO need Baroque font SMUFL_xxxx_luteDiapason4, 5, 6 ... or somesuch.
+            //      Temporary kludge, use SMUFL_EBE4_luteItalianFret4 ... .
+            fretStr.push_back(SMUFL_EBE4_luteItalianFret4 + course - 11);
+        }
+        else {
+            // courses 8..10 use slashes followed by fret letter
+            if (course >= 8) {
+                // TODO need SMUFL_xxxx_luteDiapasonSlash or 3 glyphs "/", "//", "///".
+                //      Temporary kludge, use SMUFL_E101_noteheadSlashHorizontalEnds, doesn't
+                //      look right but serves as a place holder.
+                fretStr = std::wstring(course - 7, SMUFL_E101_noteheadSlashHorizontalEnds);
+            }
+
+            static const wchar_t letter[] = {
+                SMUFL_EBC0_luteFrenchFretA,
+                SMUFL_EBC1_luteFrenchFretB,
+                SMUFL_EBC2_luteFrenchFretC,
+                SMUFL_EBC3_luteFrenchFretD,
+                SMUFL_EBC4_luteFrenchFretE,
+                SMUFL_EBC5_luteFrenchFretF,
+                SMUFL_EBC6_luteFrenchFretG,
+                SMUFL_EBC7_luteFrenchFretH,
+                SMUFL_EBC8_luteFrenchFretI,
+                SMUFL_EBC9_luteFrenchFretK,
+                SMUFL_EBCA_luteFrenchFretL,
+                SMUFL_EBCB_luteFrenchFretM,
+                SMUFL_EBCC_luteFrenchFretN,
+            };
+
+            // TODO what if fret > 12?  Some tablatures use fret p.
+            if (fret >= 0 && fret < static_cast<int>(sizeof(letter) / sizeof(letter[0]))) fretStr += letter[fret];
+        }
+        return fretStr;
+    }
+    else {
+        std::string str = StringFormat("%d", this->GetTabFret());
+        return UTF8to16(str);
+    }
+}
+
 bool Note::IsUnissonWith(Note *note, bool ignoreAccid)
 {
     if (!ignoreAccid) {
@@ -264,10 +343,10 @@ Point Note::GetStemUpSE(Doc *doc, int staffSize, bool isCueSize)
     wchar_t code = GetNoteheadGlyph(GetDrawingDur());
 
     // This is never called for now because mensural notes do not have stem/flag children
-    // For changingg this, change Note::CalcStem and Note::PrepareLayerElementParts
+    // For changing this, change Note::CalcStem and Note::PrepareLayerElementParts
     if (this->IsMensuralDur()) {
         // For mensural notation, get the code and adjust the default stem position
-        code = this->GetMensuralSmuflNoteHead();
+        code = this->GetMensuralNoteheadGlyph();
         p.y = doc->GetGlyphHeight(code, staffSize, isCueSize) / 2;
         p.x = doc->GetGlyphWidth(code, staffSize, isCueSize);
     }
@@ -298,7 +377,7 @@ Point Note::GetStemDownNW(Doc *doc, int staffSize, bool isCueSize)
     // See comment above
     if (this->IsMensuralDur()) {
         // For mensural notation, get the code and adjust the default stem position
-        code = this->GetMensuralSmuflNoteHead();
+        code = this->GetMensuralNoteheadGlyph();
         p.y = -doc->GetGlyphHeight(code, staffSize, isCueSize) / 2;
         p.x = doc->GetGlyphWidth(code, staffSize, isCueSize);
     }
@@ -315,17 +394,20 @@ Point Note::GetStemDownNW(Doc *doc, int staffSize, bool isCueSize)
     return p;
 }
 
-int Note::CalcStemLenInThirdUnits(Staff *staff)
+int Note::CalcStemLenInThirdUnits(Staff *staff, data_STEMDIRECTION stemDir)
 {
     assert(staff);
 
+    if ((stemDir != STEMDIRECTION_down) && (stemDir != STEMDIRECTION_up)) {
+        return 0;
+    }
+
     int baseStem = STANDARD_STEMLENGTH * 3;
 
     int shortening = 0;
 
-    int unitToLine = (this->GetDrawingStemDir() == STEMDIRECTION_up)
-        ? -this->GetDrawingLoc() + (staff->m_drawingLines - 1) * 2
-        : this->GetDrawingLoc();
+    int unitToLine = (stemDir == STEMDIRECTION_up) ? -this->GetDrawingLoc() + (staff->m_drawingLines - 1) * 2
+                                                   : this->GetDrawingLoc();
     if (unitToLine < 5) {
         switch (unitToLine) {
             case 4: shortening = 1; break;
@@ -339,7 +421,7 @@ int Note::CalcStemLenInThirdUnits(Staff *staff)
 
     // Limit shortening with duration shorter than quarter not when not in a beam
     if ((this->GetDrawingDur() > DUR_4) && !this->IsInBeam()) {
-        if (this->GetDrawingStemDir() == STEMDIRECTION_up) {
+        if (stemDir == STEMDIRECTION_up) {
             shortening = std::min(4, shortening);
         }
         else {
@@ -352,7 +434,7 @@ int Note::CalcStemLenInThirdUnits(Staff *staff)
     return baseStem;
 }
 
-wchar_t Note::GetMensuralSmuflNoteHead()
+wchar_t Note::GetMensuralNoteheadGlyph()
 {
     assert(this->IsMensuralDur());
 
@@ -584,7 +666,7 @@ bool Note::IsDotOverlappingWithFlag(Doc *doc, const int staffSize, bool isDotShi
     // for the purposes of vertical spacing we care only up to 16th flags - shorter ones grow upwards
     wchar_t flagGlyph = SMUFL_E242_flag16thUp;
     data_DURATION dur = this->GetDur();
-    if (dur < DURATION_16) flagGlyph = flag->GetSmuflCode(GetDrawingStemDir());
+    if (dur < DURATION_16) flagGlyph = flag->GetFlagGlyph(GetDrawingStemDir());
     const int flagHeight = doc->GetGlyphHeight(flagGlyph, staffSize, GetDrawingCueSize());
 
     const int dotMargin = flag->GetDrawingY() - GetDrawingY() - flagHeight - GetDrawingRadius(doc) / 2
@@ -593,116 +675,6 @@ bool Note::IsDotOverlappingWithFlag(Doc *doc, const int staffSize, bool isDotShi
     return dotMargin < 0;
 }
 
-std::pair<int, bool> Note::CalcNoteHorizontalOverlap(
-    Doc *doc, const std::vector<LayerElement *> &otherElements, bool isChordElement, bool isLowerElement, bool unison)
-{
-    Staff *staff = vrv_cast<Staff *>(this->GetFirstAncestor(STAFF));
-    assert(staff);
-
-    bool isInUnison = false;
-    int shift = 0;
-
-    for (int i = 0; i < int(otherElements.size()); ++i) {
-        int verticalMargin = 0;
-        int horizontalMargin = 2 * doc->GetDrawingStemWidth(staff->m_drawingStaffSize);
-        bool isUnisonElement = false;
-        if (Is(NOTE) && otherElements.at(i)->Is(NOTE)) {
-            Note *previousNote = vrv_cast<Note *>(otherElements.at(i));
-            assert(previousNote);
-            isUnisonElement = IsUnissonWith(previousNote, true);
-            // Unisson, look at the duration for the note heads
-            if (unison && IsUnissonWith(previousNote, false)) {
-                int previousDuration = previousNote->GetDrawingDur();
-                const bool isPreviousCoord = previousNote->GetParent()->Is(CHORD);
-                bool isEdgeElement = false;
-                if (isPreviousCoord) {
-                    Chord *parentChord = vrv_cast<Chord *>(previousNote->GetParent());
-                    data_STEMDIRECTION stemDir = GetDrawingStemDir();
-                    previousDuration = parentChord->GetDur();
-                    isEdgeElement = ((STEMDIRECTION_down == stemDir) && (parentChord->GetBottomNote() == previousNote))
-                        || ((STEMDIRECTION_up == stemDir) && (parentChord->GetTopNote() == previousNote));
-                }
-                // Reduce the margin to 0 for whole notes unisson
-                else if ((GetDrawingDur() == DUR_1) && (previousDuration == DUR_1)) {
-                    horizontalMargin = 0;
-                }
-                if (!isPreviousCoord || isEdgeElement || isChordElement) {
-                    if ((GetDrawingDur() == DUR_2) && (previousDuration == DUR_2)) {
-                        isInUnison = true;
-                        continue;
-                    }
-                    else if ((GetDrawingDur() > DUR_2) && (previousDuration > DUR_2)) {
-                        isInUnison = true;
-                        continue;
-                    }
-                }
-                else {
-                    horizontalMargin *= -1;
-                }
-            }
-            else if (previousNote->GetDrawingLoc() - GetDrawingLoc() > 1) {
-                continue;
-            }
-            else if (previousNote->GetDrawingLoc() - GetDrawingLoc() == 1) {
-                horizontalMargin = 0;
-            }
-            else if ((previousNote->GetDrawingLoc() - GetDrawingLoc() < 0)
-                && (previousNote->GetDrawingStemDir() != GetDrawingStemDir()) /* && !isChordElement*/) {
-                if (previousNote->GetDrawingLoc() - GetDrawingLoc() == -1) {
-                    horizontalMargin *= -1;
-                }
-                else if ((GetDrawingDur() <= DUR_1) && (previousNote->GetDrawingDur() <= DUR_1)) {
-                    continue;
-                }
-                else if (previousNote->m_crossStaff || m_crossStaff)
-                    continue;
-                else {
-                    horizontalMargin *= -1;
-                    verticalMargin = horizontalMargin;
-                }
-            }
-        }
-
-        // Nothing to do if we have no vertical overlap
-        if (!VerticalSelfOverlap(otherElements.at(i), verticalMargin)) continue;
-
-        // Nothing to do either if we have no horizontal overlap
-        if (!HorizontalSelfOverlap(otherElements.at(i), horizontalMargin + shift)) continue;
-
-        if (horizontalMargin < 0 || isLowerElement) {
-            shift -= HorizontalRightOverlap(otherElements.at(i), doc, -shift, verticalMargin);
-            if (!isUnisonElement) shift -= horizontalMargin;
-        }
-        else if ((horizontalMargin >= 0) || isChordElement) {
-            shift += HorizontalLeftOverlap(otherElements.at(i), doc, horizontalMargin - shift, verticalMargin);
-
-            // Make additional adjustments for cross-staff and unison notes
-            if (m_crossStaff) shift -= horizontalMargin;
-            if (isInUnison) shift *= -1;
-        }
-        else {
-            // Otherwise move the appropriate parent to the right
-            shift -= horizontalMargin
-                - HorizontalRightOverlap(otherElements.at(i), doc, horizontalMargin - shift, verticalMargin);
-        }
-    }
-
-    // If note is not in unison, has accidental and were to be shifted to the right - shift it to the left
-    // That way accidental will be near note that actually has accidental and not near lowest-layer note
-    if (isChordElement && unison && GetDrawingAccid() && (shift > 0)) shift *= -1;
-
-    return { shift, isInUnison };
-}
-
-void Note::AdjustOverlappingLayers(Doc *doc, const std::vector<LayerElement *> &otherElements, bool &isUnison)
-{
-    if (GetParent()->Is(CHORD)) return;
-
-    auto [margin, isInUnison] = CalcNoteHorizontalOverlap(doc, otherElements, false);
-    isUnison = isInUnison;
-    if (!isInUnison) SetDrawingXRel(GetDrawingXRel() + margin);
-}
-
 //----------------------------------------------------------------------------
 // Functors methods
 //----------------------------------------------------------------------------
@@ -763,6 +735,54 @@ int Note::ConvertMarkupAnalytical(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
+int Note::CalcArtic(FunctorParams *functorParams)
+{
+    CalcArticParams *params = vrv_params_cast<CalcArticParams *>(functorParams);
+    assert(params);
+
+    if (this->IsChordTone()) return FUNCTOR_CONTINUE;
+
+    params->m_parent = this;
+    params->m_stemDir = this->GetDrawingStemDir();
+
+    Staff *staff = vrv_cast<Staff *>(this->GetFirstAncestor(STAFF));
+    assert(staff);
+    Layer *layer = vrv_cast<Layer *>(this->GetFirstAncestor(LAYER));
+    assert(layer);
+
+    params->m_staffAbove = staff;
+    params->m_staffBelow = staff;
+    params->m_layerAbove = layer;
+    params->m_layerBelow = layer;
+    params->m_crossStaffAbove = false;
+    params->m_crossStaffBelow = false;
+
+    if (this->m_crossStaff) {
+        params->m_staffAbove = this->m_crossStaff;
+        params->m_staffBelow = this->m_crossStaff;
+        params->m_layerAbove = this->m_crossLayer;
+        params->m_layerBelow = this->m_crossLayer;
+        params->m_crossStaffAbove = true;
+        params->m_crossStaffBelow = true;
+    }
+
+    return FUNCTOR_CONTINUE;
+}
+
+int Note::AdjustArtic(FunctorParams *functorParams)
+{
+    AdjustArticParams *params = vrv_params_cast<AdjustArticParams *>(functorParams);
+    assert(params);
+
+    if (this->IsChordTone()) return FUNCTOR_CONTINUE;
+
+    params->m_parent = this;
+    params->m_articAbove.clear();
+    params->m_articBelow.clear();
+
+    return FUNCTOR_CONTINUE;
+}
+
 int Note::CalcStem(FunctorParams *functorParams)
 {
     CalcStemParams *params = vrv_params_cast<CalcStemParams *>(functorParams);
@@ -772,7 +792,7 @@ int Note::CalcStem(FunctorParams *functorParams)
         return FUNCTOR_SIBLINGS;
     }
 
-    // Stems have been calculated previously in Beam or FTrem - siblings becasue flags do not need to
+    // Stems have been calculated previously in Beam or fTrem - siblings because flags do not need to
     // be processed either
     if (this->IsInBeam() || this->IsInFTrem()) {
         return FUNCTOR_SIBLINGS;
@@ -850,44 +870,68 @@ int Note::CalcChordNoteHeads(FunctorParams *functorParams)
     Staff *staff = vrv_cast<Staff *>(this->GetFirstAncestor(STAFF));
     assert(staff);
 
-    // Nothing to do for notes that are not in a cluster
-    if (!this->m_cluster) return FUNCTOR_SIBLINGS;
+    bool mixedCue = false;
+    if (Chord *chord = this->IsChordTone(); chord != NULL) {
+        mixedCue = (chord->GetDrawingCueSize() != this->GetDrawingCueSize());
+    }
 
-    if (this->m_crossStaff) staff = this->m_crossStaff;
+    // Nothing to do for notes that are not in a cluster and without cue mixing
+    if (!m_cluster && !mixedCue) return FUNCTOR_SIBLINGS;
+
+    if (m_crossStaff) staff = m_crossStaff;
 
     int staffSize = staff->m_drawingStaffSize;
 
-    int radius = this->GetDrawingRadius(params->m_doc);
+    int diameter = 2 * this->GetDrawingRadius(params->m_doc);
+
+    // If chord consists partially of cue notes we may have to shift the noteheads
+    int cueShift = 0;
+    if (mixedCue && (this->GetDrawingStemDir() == STEMDIRECTION_up)) {
+        const double cueScaling = params->m_doc->GetCueScaling();
+        assert(cueScaling > 0.0);
+
+        if (this->GetDrawingCueSize()) {
+            // Note is cue and chord is not
+            cueShift = (1.0 / cueScaling - 1.0) * diameter; // shift to the right
+        }
+        else {
+            // Chord is cue and note is not
+            cueShift = (cueScaling - 1.0) * diameter; // shift to the left
+        }
+    }
 
     /************** notehead direction **************/
 
     bool flippedNotehead = false;
 
     // if the note is clustered, calculations are different
-    if (this->GetDrawingStemDir() == STEMDIRECTION_down) {
-        // stem down/even cluster = noteheads start on left (incorrect side)
-        if (this->m_cluster->size() % 2 == 0) {
-            flippedNotehead = (this->m_clusterPosition % 2 != 0);
+    if (m_cluster) {
+        if (this->GetDrawingStemDir() == STEMDIRECTION_down) {
+            // stem down/even cluster = noteheads start on left (incorrect side)
+            if (m_cluster->size() % 2 == 0) {
+                flippedNotehead = (m_clusterPosition % 2 != 0);
+            }
+            // else they start on normal side
+            else {
+                flippedNotehead = (m_clusterPosition % 2 == 0);
+            }
         }
-        // else they start on normal side
         else {
-            flippedNotehead = (this->m_clusterPosition % 2 == 0);
+            // flipped noteheads start on normal side no matter what
+            flippedNotehead = (m_clusterPosition % 2 == 0);
         }
     }
-    else {
-        // flipped noteheads start on normal side no matter what
-        flippedNotehead = (this->m_clusterPosition % 2 == 0);
-    }
 
     // positions notehead
     if (flippedNotehead) {
         if (this->GetDrawingStemDir() == STEMDIRECTION_up) {
-            this->SetDrawingXRel(2 * radius - params->m_doc->GetDrawingStemWidth(staffSize));
+            this->SetDrawingXRel(diameter - params->m_doc->GetDrawingStemWidth(staffSize));
         }
         else {
-            this->SetDrawingXRel(-2 * radius + params->m_doc->GetDrawingStemWidth(staffSize));
+            this->SetDrawingXRel(-diameter + params->m_doc->GetDrawingStemWidth(staffSize));
         }
     }
+    this->SetDrawingXRel(this->GetDrawingXRel() + cueShift);
 
     this->SetFlippedNotehead(flippedNotehead);
 
@@ -949,7 +993,15 @@ int Note::CalcDots(FunctorParams *functorParams)
 
         // if it's on a staff line to start with, we need to compensate here and add a full unit like DrawDots would
         const bool isDotShifted(loc % 2 == 0);
-        if (isDotShifted) ++loc;
+        if (isDotShifted) {
+            Staff *staff = vrv_cast<Staff *>(this->GetFirstAncestor(STAFF));
+            if ((GetDrawingStemDir() == STEMDIRECTION_up) || (staff->GetChildCount(LAYER) == 1))
+                ++loc;
+            else
+                --loc;
+        }
+
+        loc = CorrectDotsPlacement(staff, GetDrawingLoc(), loc, isDotShifted);
         dotLocs->push_back(loc);
 
         // Stem up, shorter than 4th and not in beam
@@ -966,6 +1018,53 @@ int Note::CalcDots(FunctorParams *functorParams)
     return FUNCTOR_SIBLINGS;
 }
 
+int Note::CorrectDotsPlacement(Staff *staff, int noteLoc, int dotLoc, bool isDotShifted)
+{
+    if (staff->GetChildCount(LAYER) > 2) return dotLoc;
+
+    ListOfObjects objects;
+    ClassIdsComparison cmp({ DOTS, NOTE });
+    Alignment *alignment = GetAlignment();
+    alignment->FindAllDescendantByComparison(&objects, &cmp, 2);
+
+    // process all dots and notes in the alignment and save them separately - notes as vector and dot locations as set
+    std::set<int> dotLocations;
+    std::vector<Note *> otherNotes;
+    for (const auto element : objects) {
+        if (element->Is(DOTS)) {
+            std::list<int> *dotLocs = vrv_cast<Dots *>(element)->GetDotLocsForStaff(staff);
+            if (dotLocs->empty()) continue;
+            std::copy(dotLocs->begin(), dotLocs->end(), std::inserter(dotLocations, dotLocations.begin()));
+        }
+        else {
+            if (this == element) continue;
+            Note *note = vrv_cast<Note *>(element);
+            otherNotes.push_back(note);
+        }
+    }
+
+    int newLocation = dotLoc;
+    for (Note *note : otherNotes) {
+        if (IsUnissonWith(note)) {
+            if (note->HasDots()) {
+                return isDotShifted ? noteLoc + 1 : dotLoc;
+            }
+            continue;
+        }
+        if (note->GetDrawingLoc() == newLocation) {
+            // mirror dot location (if it was placed above - try placing it below now)
+            newLocation = 2 * noteLoc - dotLoc;
+        }
+    }
+
+    // if there already exists a dot on the preferred location
+    if (dotLocations.find(newLocation) != dotLocations.end()) {
+        newLocation = 2 * noteLoc - dotLoc;
+    }
+
+    return newLocation;
+}
+
 int Note::CalcLedgerLines(FunctorParams *functorParams)
 {
     FunctorDocParams *params = vrv_params_cast<FunctorDocParams *>(functorParams);
@@ -1041,7 +1140,7 @@ int Note::PrepareLayerElementParts(FunctorParams *functorParams)
         currentStem->AttGraced::operator=(*this);
         currentStem->AttStems::operator=(*this);
         currentStem->AttStemsCmn::operator=(*this);
-        if (this->GetActualDur() < DUR_2) {
+        if (this->GetActualDur() < DUR_2 || (this->GetStemVisible() == BOOLEAN_false)) {
             currentStem->IsVirtual(true);
         }
     }
@@ -1112,19 +1211,6 @@ int Note::PrepareLyrics(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
-int Note::PreparePointersByLayer(FunctorParams *functorParams)
-{
-    // Call parent one too
-    LayerElement::PreparePointersByLayer(functorParams);
-
-    PreparePointersByLayerParams *params = vrv_params_cast<PreparePointersByLayerParams *>(functorParams);
-    assert(params);
-
-    params->m_currentNote = this;
-
-    return FUNCTOR_CONTINUE;
-}
-
 int Note::ResetDrawing(FunctorParams *functorParams)
 {
     // Call parent one too
