@@ -175,7 +175,7 @@ void Page::LayOutTranscription(bool force)
     AlignVerticallyParams alignVerticallyParams(doc, &alignVertically, &alignVerticallyEnd);
     this->Process(&alignVertically, &alignVerticallyParams, &alignVerticallyEnd);
 
-    // Set the pitch / pos alignement
+    // Set the pitch / pos alignment
     SetAlignmentPitchPosParams setAlignmentPitchPosParams(doc);
     Functor setAlignmentPitchPos(&Object::SetAlignmentPitchPos);
     this->Process(&setAlignmentPitchPos, &setAlignmentPitchPosParams);
@@ -267,7 +267,7 @@ void Page::LayOutHorizontally()
         this->Process(&setAlignmentX, &setAlignmentXPosParams);
     }
 
-    // Set the pitch / pos alignement
+    // Set the pitch / pos alignment
     SetAlignmentPitchPosParams setAlignmentPitchPosParams(doc);
     Functor setAlignmentPitchPos(&Object::SetAlignmentPitchPos);
     this->Process(&setAlignmentPitchPos, &setAlignmentPitchPosParams);
@@ -290,6 +290,11 @@ void Page::LayOutHorizontally()
     Functor calcDots(&Object::CalcDots);
     this->Process(&calcDots, &calcDotsParams);
 
+    // Adjust the position of outside articulations
+    CalcArticParams calcArticParams(doc);
+    Functor calcArtic(&Object::CalcArtic);
+    this->Process(&calcArtic, &calcArticParams);
+
     // Render it for filling the bounding box
     View view;
     view.SetDoc(doc);
@@ -298,12 +303,30 @@ void Page::LayOutHorizontally()
     view.SetPage(this->GetIdx(), false);
     view.DrawCurrentPage(&bBoxDC, false);
 
+    // Adjust the position of outside articulations
+    AdjustArticParams adjustArticParams(doc);
+    Functor adjustArtic(&Object::AdjustArtic);
+    this->Process(&adjustArtic, &adjustArticParams);
+
     // Adjust the x position of the LayerElement where multiple layer collide
     // Look at each LayerElement and change the m_xShift if the bounding box is overlapping
+    // For the first iteration align elements without taking dots into consideration
     Functor adjustLayers(&Object::AdjustLayers);
     AdjustLayersParams adjustLayersParams(doc, &adjustLayers, doc->m_mdivScoreDef.GetStaffNs());
     this->Process(&adjustLayers, &adjustLayersParams);
 
+    // Adjust dots for the multiple layers. Try to align dots that can be grouped together when layers collide,
+    // otherwise keep their relative positioning
+    Functor adjustDots(&Object::AdjustDots);
+    Functor adjustDotsEnd(&Object::AdjustDotsEnd);
+    AdjustDotsParams adjustDotsParams(doc, &adjustDots, &adjustDotsEnd, doc->m_mdivScoreDef.GetStaffNs());
+    this->Process(&adjustDots, &adjustDotsParams, &adjustDotsEnd);
+
+    // adjust Layers again, this time including dots positioning
+    AdjustLayersParams newAdjustLayersParams(doc, &adjustLayers, doc->m_mdivScoreDef.GetStaffNs());
+    newAdjustLayersParams.m_ignoreDots = false;
+    this->Process(&adjustLayers, &newAdjustLayersParams);
+
     // Adjust the X position of the accidentals, including in chords
     Functor adjustAccidX(&Object::AdjustAccidX);
     AdjustAccidXParams adjustAccidXParams(doc, &adjustAccidX);
@@ -314,6 +337,15 @@ void Page::LayOutHorizontally()
     Functor adjustXPos(&Object::AdjustXPos);
     Functor adjustXPosEnd(&Object::AdjustXPosEnd);
     AdjustXPosParams adjustXPosParams(doc, &adjustXPos, &adjustXPosEnd, doc->m_mdivScoreDef.GetStaffNs());
+    adjustXPosParams.m_excludes.push_back(TABDURSYM);
+    this->Process(&adjustXPos, &adjustXPosParams, &adjustXPosEnd);
+
+    // Adjust tabRhyhtm separately
+    adjustXPosParams.m_excludes.clear();
+    adjustXPosParams.m_includes.push_back(TABDURSYM);
+    adjustXPosParams.m_includes.push_back(BARLINE_ATTR_RIGHT);
+    adjustXPosParams.m_includes.push_back(METERSIG);
+    adjustXPosParams.m_includes.push_back(KEYSIG);
     this->Process(&adjustXPos, &adjustXPosParams, &adjustXPosEnd);
 
     // Adjust the X shift of the Alignment looking at the bounding boxes
@@ -324,6 +356,12 @@ void Page::LayOutHorizontally()
         doc, &adjustGraceXPos, &adjustGraceXPosEnd, doc->m_mdivScoreDef.GetStaffNs());
     this->Process(&adjustGraceXPos, &adjustGraceXPosParams, &adjustGraceXPosEnd);
 
+    // Adjust the spacing of clef changes since they are skipped in AdjustXPos
+    // Look at each clef change and  move them to the left and add space if necessary
+    Functor adjustClefChanges(&Object::AdjustClefChanges);
+    AdjustClefsParams adjustClefChangesParams(doc);
+    this->Process(&adjustClefChanges, &adjustClefChangesParams);
+
     // We need to populate processing lists for processing the document by Layer (for matching @tie) and
     // by Verse (for matching syllable connectors)
     PrepareProcessingListsParams prepareProcessingListsParams;
@@ -343,6 +381,11 @@ void Page::LayOutHorizontally()
     AdjustArpegParams adjustArpegParams(doc, &adjustArpeg);
     this->Process(&adjustArpeg, &adjustArpegParams, &adjustArpegEnd);
 
+    // Adjust the tempo
+    Functor adjustTempo(&Object::AdjustTempo);
+    AdjustTempoParams adjustTempoParams(doc);
+    this->Process(&adjustTempo, &adjustTempoParams);
+
     // Adjust the position of the tuplets
     FunctorDocParams adjustTupletsXParams(doc);
     Functor adjustTupletsX(&Object::AdjustTupletsX);
@@ -386,11 +429,6 @@ void Page::LayOutVertically()
     AlignVerticallyParams alignVerticallyParams(doc, &alignVertically, &alignVerticallyEnd);
     this->Process(&alignVertically, &alignVerticallyParams, &alignVerticallyEnd);
 
-    // Adjust the position of outside articulations
-    FunctorDocParams calcArticParams(doc);
-    Functor calcArtic(&Object::CalcArtic);
-    this->Process(&calcArtic, &calcArticParams);
-
     // Render it for filling the bounding box
     View view;
     BBoxDeviceContext bBoxDC(&view, 0, 0);
@@ -422,6 +460,10 @@ void Page::LayOutVertically()
 
     // If slurs were adjusted we need to redraw to adjust the bounding boxes
     if (adjustSlursParams.m_adjusted) {
+        // There is a problem here with cross-staff slurs: if they have been adjusted, the
+        // Slur::m_isCrossStaff flag will trigger View::DrawSlurInitial to be called again.
+        // The slur will then remain not adjusted. It will again when AdjustSlurs is called below,
+        // but in between, we can have wrong collisions detections. To be improved
         view.SetPage(this->GetIdx(), false);
         view.DrawCurrentPage(&bBoxDC, false);
     }
@@ -432,12 +474,12 @@ void Page::LayOutVertically()
     Functor setOverflowBBoxesEnd(&Object::SetOverflowBBoxesEnd);
     this->Process(&setOverflowBBoxes, &setOverflowBBoxesParams, &setOverflowBBoxesEnd);
 
-    // Adjust the positioners of floationg elements (slurs, hairpin, dynam, etc)
+    // Adjust the positioners of floating elements (slurs, hairpin, dynam, etc)
     Functor adjustFloatingPositioners(&Object::AdjustFloatingPositioners);
     AdjustFloatingPositionersParams adjustFloatingPositionersParams(doc, &adjustFloatingPositioners);
     this->Process(&adjustFloatingPositioners, &adjustFloatingPositionersParams);
 
-    // Adjust the overlap of the staff aligmnents by looking at the overflow bounding boxes params.clear();
+    // Adjust the overlap of the staff aligments by looking at the overflow bounding boxes params.clear();
     Functor adjustStaffOverlap(&Object::AdjustStaffOverlap);
     AdjustStaffOverlapParams adjustStaffOverlapParams(&adjustStaffOverlap);
     this->Process(&adjustStaffOverlap, &adjustStaffOverlapParams);
@@ -448,7 +490,7 @@ void Page::LayOutVertically()
     AdjustYPosParams adjustYPosParams(doc, &adjustYPos);
     this->Process(&adjustYPos, &adjustYPosParams);
 
-    // Adjust the positioners of floationg elements placed between staves
+    // Adjust the positioners of floating elements placed between staves
     Functor adjustFloatingPositionersBetween(&Object::AdjustFloatingPositionersBetween);
     AdjustFloatingPositionersBetweenParams adjustFloatingPositionersBetweenParams(
         doc, &adjustFloatingPositionersBetween);
@@ -459,6 +501,13 @@ void Page::LayOutVertically()
     FunctorDocParams adjustCrossStaffYPosParams(doc);
     this->Process(&adjustCrossStaffYPos, &adjustCrossStaffYPosParams, &adjustCrossStaffYPosEnd);
 
+    // Redraw are re-adjust the position of the slurs when we have cross-staff ones
+    if (adjustSlursParams.m_crossStaffSlurs) {
+        view.SetPage(this->GetIdx(), false);
+        view.DrawCurrentPage(&bBoxDC, false);
+        this->Process(&adjustSlurs, &adjustSlursParams);
+    }
+
     if (this->GetHeader()) {
         this->GetHeader()->AdjustRunningElementYPos();
     }
@@ -517,7 +566,7 @@ void Page::JustifyVertically()
         return;
     }
 
-    // Vertical justificaiton is not enabled
+    // Vertical justification is not enabled
     if (!doc->GetOptions()->m_justifyVertically.GetValue()) {
         return;
     }
@@ -564,7 +613,7 @@ void Page::LayOutPitchPos()
     // Make sure we have the correct page
     assert(this == doc->GetDrawingPage());
 
-    // Set the pitch / pos alignement
+    // Set the pitch / pos alignment
     SetAlignmentPitchPosParams setAlignmentPitchPosParams(doc);
     Functor setAlignmentPitchPos(&Object::SetAlignmentPitchPos);
     this->Process(&setAlignmentPitchPos, &setAlignmentPitchPosParams);
@@ -677,7 +726,7 @@ int Page::ApplyPPUFactor(FunctorParams *functorParams)
 
 int Page::ResetVerticalAlignment(FunctorParams *functorParams)
 {
-    // Same functor, but we have not FunctorParams so we just re-instanciate it
+    // Same functor, but we have not FunctorParams so we just re-instantiate it
     Functor resetVerticalAlignment(&Object::ResetVerticalAlignment);
 
     RunningElement *header = this->GetHeader();
