@@ -29,7 +29,7 @@
 namespace vrv {
 
 typedef std::map<RestLayer,
-    std::map<RestAccidental, std::map<RestLayerPlace, std::map<RestNotePlace, std::map<int, int> > > > >
+    std::map<RestAccidental, std::map<RestLayerPlace, std::map<RestNotePlace, std::map<int, int>>>>>
     RestOffsets;
 
 RestOffsets g_defaultRests{
@@ -118,17 +118,17 @@ RestOffsets g_defaultRests{
         { { RA_none,
             { { RLP_restOnTopLayer,
                   { { RNP_noteInSpace,
-                        { { DUR_1, -1 }, { DUR_2, 1 }, { DUR_4, 3 }, { DUR_8, 1 }, { DUR_16, 3 }, { DUR_32, 3 },
+                        { { DUR_1, -1 }, { DUR_2, 1 }, { DUR_4, 1 }, { DUR_8, 1 }, { DUR_16, 3 }, { DUR_32, 3 },
                             { DUR_64, 5 }, { DUR_128, 5 }, { DUR_LG, 3 }, { DUR_BR, 1 } } },
                       { RNP_noteOnLine,
                           { { DUR_1, 0 }, { DUR_2, 0 }, { DUR_4, 2 }, { DUR_8, 2 }, { DUR_16, 2 }, { DUR_32, 2 },
                               { DUR_64, 4 }, { DUR_128, 4 }, { DUR_LG, 2 }, { DUR_BR, 2 } } } } },
                 { RLP_restOnBottomLayer,
                     { { RNP_noteInSpace,
-                          { { DUR_1, -3 }, { DUR_2, -1 }, { DUR_4, -3 }, { DUR_8, -1 }, { DUR_16, -1 }, { DUR_32, -3 },
+                          { { DUR_1, -3 }, { DUR_2, -1 }, { DUR_4, -1 }, { DUR_8, -1 }, { DUR_16, -1 }, { DUR_32, -3 },
                               { DUR_64, -3 }, { DUR_128, -5 }, { DUR_LG, -3 }, { DUR_BR, -3 } } },
                         { RNP_noteOnLine,
-                            { { DUR_1, -2 }, { DUR_2, -2 }, { DUR_4, -4 }, { DUR_8, -2 }, { DUR_16, -2 },
+                            { { DUR_1, -2 }, { DUR_2, -2 }, { DUR_4, -2 }, { DUR_8, -2 }, { DUR_16, -2 },
                                 { DUR_32, -4 }, { DUR_64, -4 }, { DUR_128, -6 }, { DUR_LG, -2 },
                                 { DUR_BR, -2 } } } } } } } } }
 };
@@ -149,13 +149,22 @@ RestAccidental MeiAccidentalToRestAccidental(data_ACCIDENTAL_WRITTEN accidental)
 // Rest
 //----------------------------------------------------------------------------
 
+static ClassRegistrar<Rest> s_factory("rest", REST);
+
 Rest::Rest()
-    : LayerElement("rest-"), DurationInterface(), PositionInterface(), AttColor(), AttCue(), AttRestVisMensural()
+    : LayerElement("rest-")
+    , DurationInterface()
+    , PositionInterface()
+    , AttColor()
+    , AttCue()
+    , AttExtSym()
+    , AttRestVisMensural()
 {
     RegisterInterface(DurationInterface::GetAttClasses(), DurationInterface::IsInterface());
     RegisterInterface(PositionInterface::GetAttClasses(), PositionInterface::IsInterface());
     RegisterAttClass(ATT_COLOR);
     RegisterAttClass(ATT_CUE);
+    RegisterAttClass(ATT_EXTSYM);
     RegisterAttClass(ATT_RESTVISMENSURAL);
     Reset();
 }
@@ -169,6 +178,7 @@ void Rest::Reset()
     PositionInterface::Reset();
     ResetColor();
     ResetCue();
+    ResetExtSym();
     ResetRestVisMensural();
 }
 
@@ -210,33 +220,33 @@ void Rest::AddChild(Object *child)
 
 wchar_t Rest::GetRestGlyph() const
 {
-    int symc = 0;
-    switch (this->GetActualDur()) {
-        case DUR_LG: symc = SMUFL_E4E1_restLonga; break;
-        case DUR_BR: symc = SMUFL_E4E2_restDoubleWhole; break;
-        case DUR_1: symc = SMUFL_E4E3_restWhole; break;
-        case DUR_2: symc = SMUFL_E4E4_restHalf; break;
-        case DUR_4: symc = SMUFL_E4E5_restQuarter; break;
-        case DUR_8: symc = SMUFL_E4E6_rest8th; break;
-        case DUR_16: symc = SMUFL_E4E7_rest16th; break;
-        case DUR_32: symc = SMUFL_E4E8_rest32nd; break;
-        case DUR_64: symc = SMUFL_E4E9_rest64th; break;
-        case DUR_128: symc = SMUFL_E4EA_rest128th; break;
-        case DUR_256: symc = SMUFL_E4EB_rest256th; break;
-        case DUR_512: symc = SMUFL_E4EC_rest512th; break;
-        case DUR_1024: symc = SMUFL_E4ED_rest1024th; break;
+    // If there is glyph.num, prioritize it
+    if (HasGlyphNum()) {
+        wchar_t code = GetGlyphNum();
+        if (NULL != Resources::GetGlyph(code)) return code;
     }
-    return symc;
-}
-
-int Rest::GetRestLocOffset(int loc)
-{
-    switch (this->GetActualDur()) {
-        case DUR_1: loc += 2; break;
-        default: loc += 0; break;
+    // If there is glyph.name (second priority)
+    else if (HasGlyphName()) {
+        wchar_t code = Resources::GetGlyphCode(GetGlyphName());
+        if (NULL != Resources::GetGlyph(code)) return code;
     }
 
-    return loc;
+    switch (this->GetActualDur()) {
+        case DUR_LG: return SMUFL_E4E1_restLonga; break;
+        case DUR_BR: return SMUFL_E4E2_restDoubleWhole; break;
+        case DUR_1: return SMUFL_E4E3_restWhole; break;
+        case DUR_2: return SMUFL_E4E4_restHalf; break;
+        case DUR_4: return SMUFL_E4E5_restQuarter; break;
+        case DUR_8: return SMUFL_E4E6_rest8th; break;
+        case DUR_16: return SMUFL_E4E7_rest16th; break;
+        case DUR_32: return SMUFL_E4E8_rest32nd; break;
+        case DUR_64: return SMUFL_E4E9_rest64th; break;
+        case DUR_128: return SMUFL_E4EA_rest128th; break;
+        case DUR_256: return SMUFL_E4EB_rest256th; break;
+        case DUR_512: return SMUFL_E4EC_rest512th; break;
+        case DUR_1024: return SMUFL_E4ED_rest1024th; break;
+    }
+    return 0;
 }
 
 void Rest::UpdateFromTransLoc(const TransPitch &tp)
@@ -258,13 +268,16 @@ int Rest::GetOptimalLayerLocation(Staff *staff, Layer *layer, int defaultLocatio
     // handle rest positioning for 2 layers. 3 layers and more are much more complex to solve
     if (layerCount != 2) return defaultLocation;
 
+    Staff *realStaff = m_crossStaff ? m_crossStaff : staff;
+
     ListOfObjects layers;
     ClassIdComparison matchType(LAYER);
-    staff->FindAllDescendantByComparison(&layers, &matchType);
-    const bool isTopLayer(vrv_cast<Layer *>(*layers.begin())->GetN() == layer->GetN());
+    realStaff->FindAllDescendantByComparison(&layers, &matchType);
+    const bool isTopLayer((m_crossStaff && (staff->GetN() < m_crossStaff->GetN()))
+        || (!m_crossStaff && vrv_cast<Layer *>(*layers.begin())->GetN() == layer->GetN()));
 
     // find best rest location relative to elements on other layers
-    const auto otherLayerRelativeLocationInfo = GetLocationRelativeToOtherLayers(layers, layer);
+    const auto otherLayerRelativeLocationInfo = GetLocationRelativeToOtherLayers(layers, layer, isTopLayer);
     int currentLayerRelativeLocation = GetLocationRelativeToCurrentLayer(staff, layer, isTopLayer);
     int otherLayerRelativeLocation = otherLayerRelativeLocationInfo.first
         + GetRestOffsetFromOptions(RL_otherLayer, otherLayerRelativeLocationInfo, isTopLayer);
@@ -276,21 +289,38 @@ int Rest::GetOptimalLayerLocation(Staff *staff, Layer *layer, int defaultLocatio
         currentLayerRelativeLocation
             += GetRestOffsetFromOptions(RL_sameLayer, currentLayerRelativeLocationInfo, isTopLayer);
     }
+    if (m_crossStaff) {
+        if (isTopLayer) {
+            otherLayerRelativeLocation += defaultLocation + 2;
+        }
+        else {
+            otherLayerRelativeLocation -= defaultLocation + 2;
+        }
+    }
 
-    return isTopLayer ? std::max({ otherLayerRelativeLocation, currentLayerRelativeLocation, defaultLocation })
-                      : std::min({ otherLayerRelativeLocation, currentLayerRelativeLocation, defaultLocation });
+    // for two layers, top layer shouldn't go below center and lower layer shouldn't go above it. Enforce this by adding
+    // margin that will adjust rest position
+    const int marginLocation = isTopLayer ? 6 : 2;
+    const int optimalLocation = isTopLayer
+        ? std::max({ otherLayerRelativeLocation, currentLayerRelativeLocation, defaultLocation, marginLocation })
+        : std::min({ otherLayerRelativeLocation, currentLayerRelativeLocation, defaultLocation, marginLocation });
+
+    return optimalLocation;
 }
 
 std::pair<int, RestAccidental> Rest::GetLocationRelativeToOtherLayers(
-    const ListOfObjects &layersList, Layer *currentLayer)
+    const ListOfObjects &layersList, Layer *currentLayer, bool isTopLayer)
 {
     if (!currentLayer) return { VRV_UNSET, RA_none };
-    const bool isTopLayer(vrv_cast<Layer *>(*layersList.begin())->GetN() == currentLayer->GetN());
 
     // Get iterator to another layer. We're going to find coliding elements there
     auto layerIter = std::find_if(layersList.begin(), layersList.end(),
         [&](Object *foundLayer) { return vrv_cast<Layer *>(foundLayer)->GetN() != currentLayer->GetN(); });
-    if (layerIter == layersList.end()) return { VRV_UNSET, RA_none };
+    if (layerIter == layersList.end()) {
+        if (!m_crossStaff) return { VRV_UNSET, RA_none };
+        // if we're dealing with cross-staff item, get first/last layer, depending whether rest is on top or bottom
+        layerIter = isTopLayer ? layersList.begin() : std::prev(layersList.end());
+    }
     auto collidingElementsList = vrv_cast<Layer *>(*layerIter)->GetLayerElementsForTimeSpanOf(this);
 
     std::pair<int, RestAccidental> finalElementInfo = { VRV_UNSET, RA_none };
@@ -301,6 +331,11 @@ std::pair<int, RestAccidental> Rest::GetLocationRelativeToOtherLayers(
         //  If note on other layer is not on the same x position as rest - ignore its accidental
         if (GetAlignment()->GetTime() != vrv_cast<LayerElement *>(object)->GetAlignment()->GetTime()) {
             currentElementInfo.second = RA_none;
+            // limit how much rest can be offset when there is duration overlap, but no x position overlap
+            if ((isTopLayer && (currentElementInfo.first > 12)) || (!isTopLayer && (currentElementInfo.first < -4))) {
+                if (finalElementInfo.first != VRV_UNSET) continue;
+                currentElementInfo.first = isTopLayer ? 12 : -4;
+            }
         }
         if ((VRV_UNSET == finalElementInfo.first) || (isTopLayer && (finalElementInfo.first < currentElementInfo.first))
             || (!isTopLayer && (finalElementInfo.first > currentElementInfo.first))) {
@@ -343,10 +378,21 @@ int Rest::GetLocationRelativeToCurrentLayer(Staff *currentStaff, Layer *currentL
         ? GetElementLocation(nextElement, currentLayer, !isTopLayer).first
         : GetFirstRelativeElementLocation(currentStaff, currentLayer, false, isTopLayer);
 
-    if (VRV_UNSET == previousElementLoc) return nextElementLoc;
-    if (VRV_UNSET == nextElementLoc) return previousElementLoc;
+    int currentOptimalLocation = 0;
+    if (VRV_UNSET == previousElementLoc) {
+        currentOptimalLocation = nextElementLoc;
+    }
+    else if (VRV_UNSET == nextElementLoc) {
+        currentOptimalLocation = previousElementLoc;
+    }
+    else {
+        currentOptimalLocation = (previousElementLoc + nextElementLoc) / 2;
+    }
+    const int marginLocation = isTopLayer ? 10 : -2;
+    currentOptimalLocation = isTopLayer ? std::min(currentOptimalLocation, marginLocation)
+                                        : std::max(currentOptimalLocation, marginLocation);
 
-    return isTopLayer ? std::min(previousElementLoc, nextElementLoc) : std::max(previousElementLoc, nextElementLoc);
+    return currentOptimalLocation;
 }
 
 int Rest::GetFirstRelativeElementLocation(Staff *currentStaff, Layer *currentLayer, bool isPrevious, bool isTopLayer)
@@ -407,30 +453,74 @@ std::pair<int, RestAccidental> Rest::GetElementLocation(Object *object, Layer *l
             (accid && accid->GetAccid() != 0) ? MeiAccidentalToRestAccidental(accid->GetAccid()) : RA_none };
     }
     if (object->Is(FTREM)) {
-        std::vector<std::pair<int, RestAccidental> > btremElements;
+        std::vector<std::pair<int, RestAccidental>> btremElements;
         for (int i = 0; i < object->GetChildCount(); ++i) {
             btremElements.emplace_back(GetElementLocation(object->GetChild(i), layer, isTopLayer));
         }
         return isTopLayer ? *std::max_element(btremElements.begin(), btremElements.end())
                           : *std::min_element(btremElements.begin(), btremElements.end());
     }
+    if (object->Is(REST)) {
+        if (!m_crossStaff) return { VRV_UNSET, RA_none };
+        Rest *rest = vrv_cast<Rest *>(object);
+        assert(rest);
+        return { rest->GetDrawingLoc(), RA_none };
+    }
     return { VRV_UNSET, RA_none };
 }
 
 int Rest::GetRestOffsetFromOptions(
     RestLayer layer, const std::pair<int, RestAccidental> &location, bool isTopLayer) const
 {
+    int duration = GetActualDur();
+    if (duration > DURATION_128) duration = DURATION_128;
     return g_defaultRests.at(layer)
         .at(RL_sameLayer == layer ? location.second : RA_none)
         .at(isTopLayer ? RLP_restOnTopLayer : RLP_restOnBottomLayer)
         .at(0 == location.first % 2 ? RNP_noteOnLine : RNP_noteInSpace)
-        .at(GetActualDur());
+        .at(duration);
 }
 
 //----------------------------------------------------------------------------
 // Functors methods
 //----------------------------------------------------------------------------
 
+int Rest::AdjustBeams(FunctorParams *functorParams)
+{
+    AdjustBeamParams *params = vrv_params_cast<AdjustBeamParams *>(functorParams);
+    assert(params);
+
+    if (!params->m_beam) return FUNCTOR_SIBLINGS;
+
+    // Calculate possible overlap for the rest with beams
+    int leftMargin = 0, rightMargin = 0;
+    const int beams = vrv_cast<Beam *>(params->m_beam)->m_shortestDur - DUR_4;
+    const int beamWidth = vrv_cast<Beam *>(params->m_beam)->m_beamWidth;
+    if (params->m_directionBias > 0) {
+        leftMargin = params->m_y1 - beams * beamWidth - GetSelfTop();
+        rightMargin = params->m_y2 - beams * beamWidth - GetSelfTop();
+    }
+    else {
+        leftMargin = GetSelfBottom() - params->m_y1 - beams * beamWidth;
+        rightMargin = GetSelfBottom() - params->m_y2 - beams * beamWidth;
+    }
+
+    // Adjust drawing location for the rest based on the overlap with beams. Adjustmend should be an even number, so
+    // that rest is positioned properly
+    const int overlapMargin = std::min(leftMargin, rightMargin);
+    if (overlapMargin < 0) {
+        Staff *staff = vrv_cast<Staff *>(GetFirstAncestor(STAFF));
+        assert(staff);
+        const int unit = params->m_doc->GetDrawingUnit(staff->m_drawingStaffSize);
+        const int locAdjust = (params->m_directionBias * (overlapMargin - 2 * unit + 1) / unit);
+        const int newLoc = GetDrawingLoc() + locAdjust - locAdjust % 2;
+        SetDrawingLoc(newLoc);
+        SetDrawingYRel(staff->CalcPitchPosYRel(params->m_doc, newLoc));
+    }
+
+    return FUNCTOR_CONTINUE;
+}
+
 int Rest::ConvertMarkupAnalytical(FunctorParams *functorParams)
 {
     ConvertMarkupAnalyticalParams *params = vrv_params_cast<ConvertMarkupAnalyticalParams *>(functorParams);
