@@ -28,6 +28,7 @@
 #include "syl.h"
 #include "system.h"
 #include "timeinterface.h"
+#include "tuning.h"
 #include "verse.h"
 #include "vrv.h"
 #include "zone.h"
@@ -38,6 +39,8 @@ namespace vrv {
 // Staff
 //----------------------------------------------------------------------------
 
+static ClassRegistrar<Staff> s_factory("staff", STAFF);
+
 Staff::Staff(int n) : Object("staff-"), FacsimileInterface(), AttNInteger(), AttTyped(), AttVisibility()
 {
     RegisterAttClass(ATT_NINTEGER);
@@ -75,6 +78,7 @@ void Staff::Reset()
     m_staffAlignment = NULL;
     m_timeSpanningElements.clear();
     m_drawingStaffDef = NULL;
+    m_drawingTuning = NULL;
 
     ClearLedgerLines();
 }
@@ -94,6 +98,7 @@ void Staff::CloneReset()
     m_staffAlignment = NULL;
     m_timeSpanningElements.clear();
     m_drawingStaffDef = NULL;
+    m_drawingTuning = NULL;
 }
 
 const ArrayOfObjects *Staff::GetChildren(bool docChildren) const
@@ -215,6 +220,30 @@ bool Staff::DrawingIsVisible()
     return (staffDef->GetDrawingVisibility() != OPTIMIZATION_HIDDEN);
 }
 
+bool Staff::IsMensural()
+{
+    bool isMensural = (this->m_drawingNotationType == NOTATIONTYPE_mensural
+        || this->m_drawingNotationType == NOTATIONTYPE_mensural_white
+        || this->m_drawingNotationType == NOTATIONTYPE_mensural_black);
+    return isMensural;
+}
+
+bool Staff::IsNeume()
+{
+    bool isNeume = (this->m_drawingNotationType == NOTATIONTYPE_neume);
+    return isNeume;
+}
+
+bool Staff::IsTablature()
+{
+    bool isTablature
+        = (this->m_drawingNotationType == NOTATIONTYPE_tab || this->m_drawingNotationType == NOTATIONTYPE_tab_guitar
+            || this->m_drawingNotationType == NOTATIONTYPE_tab_lute_italian
+            || this->m_drawingNotationType == NOTATIONTYPE_tab_lute_french
+            || this->m_drawingNotationType == NOTATIONTYPE_tab_lute_german);
+    return isTablature;
+}
+
 int Staff::CalcPitchPosYRel(Doc *doc, int loc)
 {
     assert(doc);
@@ -271,6 +300,29 @@ void Staff::SetFromFacsimile(Doc *doc)
     this->AdjustDrawingStaffSize();
 }
 
+bool Staff::IsOnStaffLine(int y, Doc *doc)
+{
+    assert(doc);
+
+    return ((y - this->GetDrawingY()) % (2 * doc->GetDrawingUnit(this->m_drawingStaffSize)) == 0);
+}
+
+int Staff::GetNearestInterStaffPosition(int y, Doc *doc, data_STAFFREL place)
+{
+    assert(doc);
+
+    int yPos = y - this->GetDrawingY();
+    int distance = yPos % doc->GetDrawingUnit(this->m_drawingStaffSize);
+    if (place == STAFFREL_above) {
+        if (distance > 0) distance = doc->GetDrawingUnit(this->m_drawingStaffSize) - distance;
+        return y - distance + doc->GetDrawingUnit(this->m_drawingStaffSize);
+    }
+    else {
+        if (distance < 0) distance = doc->GetDrawingUnit(this->m_drawingStaffSize) + distance;
+        return y - distance - doc->GetDrawingUnit(this->m_drawingStaffSize);
+    }
+}
+
 //----------------------------------------------------------------------------
 // LedgerLine
 //----------------------------------------------------------------------------
@@ -291,7 +343,7 @@ void LedgerLine::AddDash(int left, int right)
 {
     assert(left < right);
 
-    std::list<std::pair<int, int> >::iterator iter;
+    std::list<std::pair<int, int>>::iterator iter;
 
     // First add the dash
     for (iter = m_dashes.begin(); iter != m_dashes.end(); ++iter) {
@@ -300,7 +352,7 @@ void LedgerLine::AddDash(int left, int right)
     m_dashes.insert(iter, std::make_pair(left, right));
 
     // Merge overlapping dashes
-    std::list<std::pair<int, int> >::iterator previous = m_dashes.begin();
+    std::list<std::pair<int, int>>::iterator previous = m_dashes.begin();
     iter = m_dashes.begin();
     ++iter;
     while (iter != m_dashes.end()) {
@@ -335,16 +387,17 @@ int Staff::ConvertToCastOffMensural(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
-int Staff::UnsetCurrentScoreDef(FunctorParams *functorParams)
+int Staff::ScoreDefUnsetCurrent(FunctorParams *functorParams)
 {
     m_drawingStaffDef = NULL;
+    m_drawingTuning = NULL;
 
     return FUNCTOR_CONTINUE;
 }
 
-int Staff::OptimizeScoreDef(FunctorParams *functorParams)
+int Staff::ScoreDefOptimize(FunctorParams *functorParams)
 {
-    OptimizeScoreDefParams *params = vrv_params_cast<OptimizeScoreDefParams *>(functorParams);
+    ScoreDefOptimizeParams *params = vrv_params_cast<ScoreDefOptimizeParams *>(functorParams);
     assert(params);
 
     assert(params->m_currentScoreDef);
@@ -352,7 +405,7 @@ int Staff::OptimizeScoreDef(FunctorParams *functorParams)
 
     if (!staffDef) {
         LogDebug(
-            "Could not find staffDef for staff (%d) when optimizing scoreDef in Staff::OptimizeScoreDef", this->GetN());
+            "Could not find staffDef for staff (%d) when optimizing scoreDef in Staff::ScoreDefOptimize", this->GetN());
         return FUNCTOR_SIBLINGS;
     }
 
@@ -370,7 +423,8 @@ int Staff::OptimizeScoreDef(FunctorParams *functorParams)
 
     // Ignore layers that are empty (or with @sameas)
     ListOfObjects layers;
-    IsEmptyComparison matchTypeLayer(LAYER, true);
+    IsEmptyComparison matchTypeLayer(LAYER);
+    matchTypeLayer.ReverseComparison();
     this->FindAllDescendantByComparison(&layers, &matchTypeLayer);
 
     ListOfObjects mRests;
