@@ -141,7 +141,7 @@ int System::GetHeight() const
 int System::GetMinimumSystemSpacing(const Doc *doc) const
 {
     const auto &spacingSystem = doc->GetOptions()->m_spacingSystem;
-    if (!spacingSystem.isSet()) {
+    if (!spacingSystem.IsSet()) {
         assert(m_drawingScoreDef);
         if (m_drawingScoreDef->HasSpacingSystem()) {
             return m_drawingScoreDef->GetSpacingSystem() * doc->GetDrawingUnit(100);
@@ -184,7 +184,7 @@ bool System::SetCurrentFloatingPositioner(
 
 void System::SetDrawingScoreDef(ScoreDef *drawingScoreDef)
 {
-    assert(!m_drawingScoreDef); // We should always call UnsetCurrentScoreDef before
+    assert(!m_drawingScoreDef); // We should always call UnscoreDefSetCurrent before
 
     m_drawingScoreDef = new ScoreDef();
     *m_drawingScoreDef = *drawingScoreDef;
@@ -193,9 +193,37 @@ void System::SetDrawingScoreDef(ScoreDef *drawingScoreDef)
 
 bool System::HasMixedDrawingStemDir(LayerElement *start, LayerElement *end)
 {
+    assert(start);
+    assert(end);
+
+    // It is too inefficient to look for chord and notes over the entire system
+    // We need first to get a list of measures
+    Object *measureStart = start->GetFirstAncestor(MEASURE);
+    assert(measureStart);
+    Object *measureEnd = end->GetFirstAncestor(MEASURE);
+    assert(measureEnd);
+    ListOfObjects measures;
+
+    // start and end are in the same measure, this is the only one we need
+    if (measureStart == measureEnd) {
+        measures.push_back(measureStart);
+    }
+    // otherwise look for a measures in between
+    else {
+        ClassIdComparison isMeasure(MEASURE);
+        Functor findAllBetween(&Object::FindAllBetween);
+        FindAllBetweenParams findAllBetweenParams(&isMeasure, &measures, measureStart, measureEnd);
+        this->Process(&findAllBetween, &findAllBetweenParams, NULL, NULL, 1);
+    }
+
+    // Now we can look for chords and note
     ClassIdsComparison matchType({ CHORD, NOTE });
     ListOfObjects children;
-    this->FindAllDescendantBetween(&children, &matchType, start, end);
+    for (auto &measure : measures) {
+        Object *curStart = (measure == measureStart) ? start : measure->GetFirst();
+        Object *curEnd = (measure == measureEnd) ? end : measure->GetLast();
+        measure->FindAllDescendantBetween(&children, &matchType, curStart, curEnd, false);
+    }
 
     Layer *layerStart = vrv_cast<Layer *>(start->GetFirstAncestor(LAYER));
     assert(layerStart);
@@ -239,7 +267,7 @@ void System::AddToDrawingListIfNeccessary(Object *object)
 
     if (!object->HasInterface(INTERFACE_TIME_SPANNING)) return;
 
-    if (object->Is({ BRACKETSPAN, FIGURE, GLISS, HAIRPIN, PHRASE, OCTAVE, SLUR, SYL, TIE })) {
+    if (object->Is({ BRACKETSPAN, FIGURE, GLISS, HAIRPIN, OCTAVE, PHRASE, PITCHINFLECTION, SLUR, SYL, TIE })) {
         this->AddToDrawingList(object);
     }
     else if (object->Is(DIR)) {
@@ -276,7 +304,7 @@ void System::AddToDrawingListIfNeccessary(Object *object)
 // System functor methods
 //----------------------------------------------------------------------------
 
-int System::UnsetCurrentScoreDef(FunctorParams *functorParams)
+int System::ScoreDefUnsetCurrent(FunctorParams *functorParams)
 {
     if (m_drawingScoreDef) {
         delete m_drawingScoreDef;
@@ -288,9 +316,9 @@ int System::UnsetCurrentScoreDef(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
-int System::OptimizeScoreDef(FunctorParams *functorParams)
+int System::ScoreDefOptimize(FunctorParams *functorParams)
 {
-    OptimizeScoreDefParams *params = vrv_params_cast<OptimizeScoreDefParams *>(functorParams);
+    ScoreDefOptimizeParams *params = vrv_params_cast<ScoreDefOptimizeParams *>(functorParams);
     assert(params);
 
     this->IsDrawingOptimized(true);
@@ -308,9 +336,9 @@ int System::OptimizeScoreDef(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
-int System::OptimizeScoreDefEnd(FunctorParams *functorParams)
+int System::ScoreDefOptimizeEnd(FunctorParams *functorParams)
 {
-    OptimizeScoreDefParams *params = vrv_params_cast<OptimizeScoreDefParams *>(functorParams);
+    ScoreDefOptimizeParams *params = vrv_params_cast<ScoreDefOptimizeParams *>(functorParams);
     assert(params);
 
     params->m_currentScoreDef->Process(params->m_functor, params, params->m_functorEnd);
@@ -319,6 +347,16 @@ int System::OptimizeScoreDefEnd(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
+int System::ScoreDefSetGrpSym(FunctorParams *functorParams)
+{
+    ScoreDefSetGrpSymParams *params = vrv_params_cast<ScoreDefSetGrpSymParams *>(functorParams);
+    assert(params);
+
+    if (m_drawingScoreDef) m_drawingScoreDef->Process(params->m_functor, functorParams);
+
+    return FUNCTOR_SIBLINGS;
+}
+
 int System::ResetHorizontalAlignment(FunctorParams *functorParams)
 {
     SetDrawingXRel(0);
@@ -531,6 +569,16 @@ int System::AdjustSylSpacingEnd(FunctorParams *functorParams)
     return FUNCTOR_CONTINUE;
 }
 
+int System::AdjustTempo(FunctorParams *functorParams)
+{
+    AdjustTempoParams *params = vrv_params_cast<AdjustTempoParams *>(functorParams);
+    assert(params);
+
+    params->m_systemAligner = &m_systemAligner;
+
+    return FUNCTOR_CONTINUE;
+}
+
 int System::AdjustYPos(FunctorParams *functorParams)
 {
     AdjustYPosParams *params = vrv_params_cast<AdjustYPosParams *>(functorParams);
@@ -589,7 +637,7 @@ int System::AlignSystems(FunctorParams *functorParams)
 
     params->m_justificationSum += m_systemAligner.GetJustificationSum(params->m_doc);
     if (!this->GetIdx()) {
-        // remove extra system justification factor to get exaclty (systemsCount-1)*justificationSystem
+        // remove extra system justification factor to get exactly (systemsCount-1)*justificationSystem
         params->m_justificationSum -= params->m_doc->GetOptions()->m_justificationSystem.GetValue();
     }
 
@@ -856,7 +904,7 @@ int System::UnCastOff(FunctorParams *functorParams)
     UnCastOffParams *params = vrv_params_cast<UnCastOffParams *>(functorParams);
     assert(params);
 
-    // Just move all the content of the system to the continous one (parameter)
+    // Just move all the content of the system to the continuous one (parameter)
     // Use the MoveChildrenFrom method that moves and relinquishes them
     // See Object::Relinquish
     params->m_currentSystem->MoveChildrenFrom(this);
