@@ -34,6 +34,8 @@ namespace vrv {
 // Tuplet
 //----------------------------------------------------------------------------
 
+static ClassRegistrar<Tuplet> s_factory("tuplet", TUPLET);
+
 Tuplet::Tuplet()
     : LayerElement("tuplet-")
     , ObjectListInterface()
@@ -150,7 +152,7 @@ void Tuplet::FilterList(ArrayOfObjects *childList)
     }
 }
 
-void Tuplet::CalcDrawingBracketAndNumPos()
+void Tuplet::CalcDrawingBracketAndNumPos(bool tupletNumHead)
 {
     m_drawingBracketPos = STAFFREL_basic_NONE;
 
@@ -205,6 +207,11 @@ void Tuplet::CalcDrawingBracketAndNumPos()
     // true means up
     m_drawingBracketPos = ups > downs ? STAFFREL_basic_above : STAFFREL_basic_below;
 
+    if (tupletNumHead) {
+        m_drawingBracketPos
+            = (m_drawingBracketPos == STAFFREL_basic_below) ? STAFFREL_basic_above : STAFFREL_basic_below;
+    }
+
     // also use it for the num unless it is already set
     if (m_drawingNumPos == STAFFREL_basic_NONE) {
         m_drawingNumPos = m_drawingBracketPos;
@@ -434,7 +441,7 @@ int Tuplet::AdjustTupletsY(FunctorParams *functorParams)
         if (beam) {
             // Check for possible articulations
             ListOfObjects artics;
-            ClassIdsComparison comparison({ ARTIC, ARTIC_PART });
+            ClassIdsComparison comparison({ ARTIC });
             this->FindAllDescendantByComparison(&artics, &comparison);
 
             int articPadding = 0;
@@ -468,9 +475,9 @@ int Tuplet::AdjustTupletsY(FunctorParams *functorParams)
                 = (m_drawingBracketPos == STAFFREL_basic_above) ? 0 : -params->m_doc->GetDrawingStaffSize(staffSize);
 
             // Check for overlap with content
-            // Possible issue with beam above the tuplet - not sure this will be noticable
+            // Possible issue with beam above the tuplet - not sure this will be noticeable
             ListOfObjects descendants;
-            ClassIdsComparison comparison({ ARTIC, ARTIC_PART, ACCID, BEAM, DOT, FLAG, NOTE, REST, STEM });
+            ClassIdsComparison comparison({ ARTIC, ACCID, BEAM, DOT, FLAG, NOTE, REST, STEM });
             this->FindAllDescendantByComparison(&descendants, &comparison);
 
             // Possible fix for beam above tuplet
@@ -527,7 +534,7 @@ int Tuplet::AdjustTupletsY(FunctorParams *functorParams)
                 tupletNum->SetDrawingYRel(yRel);
             }
 
-            // Find if there is a mix of cross-staf and non-cross-staff elements in the tuplet
+            // Find if there is a mix of cross-staff and non-cross-staff elements in the tuplet
             ListOfObjects descendants;
             ClassIdsComparison comparison({ CHORD, NOTE, REST });
             this->FindAllDescendantByComparison(&descendants, &comparison);
