@@ -214,8 +214,10 @@ SystemAligner::SpacingType SystemAligner::CalculateSpacingAbove(StaffDef *staffD
             notFirstInGroup = notFirstInGroup || (firstVisible && firstVisible != staffChild);
             if (notFirstInGroup) {
                 StaffGrp *staffGrp = dynamic_cast<StaffGrp *>(staffParent);
-                if (staffGrp && staffGrp->HasSymbol()) {
-                    switch (staffGrp->GetSymbol()) {
+                if (staffGrp && staffGrp->GetFirst(GRPSYM)) {
+                    GrpSym *grpSym = vrv_cast<GrpSym *>(staffGrp->GetFirst(GRPSYM));
+                    assert(grpSym);
+                    switch (grpSym->GetSymbol()) {
                         case staffGroupingSym_SYMBOL_brace: spacingType = SpacingType::Brace; break;
                         case staffGroupingSym_SYMBOL_bracket:
                         case staffGroupingSym_SYMBOL_bracketsq: spacingType = SpacingType::Bracket; break;
@@ -384,7 +386,7 @@ int StaffAlignment::GetMinimumStaffSpacing(const Doc *doc, const AttSpacing *att
     const auto &option = doc->GetOptions()->m_spacingStaff;
     int spacing = option.GetValue() * doc->GetDrawingUnit(GetStaffSize());
 
-    if (!option.isSet() && attSpacing->HasSpacingStaff()) {
+    if (!option.IsSet() && attSpacing->HasSpacingStaff()) {
         spacing = attSpacing->GetSpacingStaff() * doc->GetDrawingUnit(100);
     }
     return spacing;
@@ -413,13 +415,13 @@ int StaffAlignment::GetMinimumSpacing(const Doc *doc) const
                 }
                 case SystemAligner::SpacingType::Brace: {
                     const auto &option = doc->GetOptions()->m_spacingBraceGroup;
-                    spacing = option.isSet() ? option.GetValue() * doc->GetDrawingUnit(GetStaffSize())
+                    spacing = option.IsSet() ? option.GetValue() * doc->GetDrawingUnit(GetStaffSize())
                                              : GetMinimumStaffSpacing(doc, scoreDefSpacing);
                     break;
                 }
                 case SystemAligner::SpacingType::Bracket: {
                     const auto &option = doc->GetOptions()->m_spacingBracketGroup;
-                    spacing = option.isSet() ? option.GetValue() * doc->GetDrawingUnit(GetStaffSize())
+                    spacing = option.IsSet() ? option.GetValue() * doc->GetDrawingUnit(GetStaffSize())
                                              : GetMinimumStaffSpacing(doc, scoreDefSpacing);
                     break;
                 }
@@ -528,7 +530,7 @@ void StaffAlignment::ReAdjustFloatingPositionersGrps(AdjustFloatingPositionerGrp
     // The initial next position is the original position of the first group. Nothing will happen for it.
     int nextYRel = grpIdYRel.at(0).second;
 
-    // For each grpId (sorted, see above), loop to find the highest / lowest positon to put the next group
+    // For each grpId (sorted, see above), loop to find the highest / lowest position to put the next group
     // The move the next group (if not already higher or lower)
     for (auto const &grp : grpIdYRel) {
         // Check if the next group it not already higher or lower.
@@ -661,7 +663,7 @@ int StaffAlignment::AdjustFloatingPositioners(FunctorParams *functorParams)
         auto i = overflowBoxes->begin();
         auto end = overflowBoxes->end();
         while (i != end) {
-            // find all the overflowing elements from the staff that overlap horizonatally
+            // find all the overflowing elements from the staff that overlap horizontally
             i = std::find_if(i, end, [iter](BoundingBox *elem) { return (*iter)->HorizontalContentOverlap(elem); });
             if (i != end) {
                 // update the yRel accordingly
@@ -724,7 +726,7 @@ int StaffAlignment::AdjustFloatingPositionersBetween(FunctorParams *functorParam
         bool adjusted = false;
         while (i != end) {
 
-            // find all the overflowing elements from the staff that overlap horizonatally
+            // find all the overflowing elements from the staff that overlap horizontally
             i = std::find_if(
                 i, end, [positioner](BoundingBox *elem) { return positioner->HorizontalContentOverlap(elem); });
             if (i != end) {
@@ -757,7 +759,7 @@ int StaffAlignment::AdjustFloatingPositionerGrps(FunctorParams *functorParams)
     assert(params);
 
     ArrayOfFloatingPositioners positioners;
-    // make a temporary copy of positionners with a classId desired and that have a drawing grpId
+    // make a temporary copy of positioners with a classId desired and that have a drawing grpId
     std::copy_if(m_floatingPositioners.begin(), m_floatingPositioners.end(), std::back_inserter(positioners),
         [params](FloatingPositioner *positioner) {
             assert(positioner->GetObject());
@@ -766,7 +768,7 @@ int StaffAlignment::AdjustFloatingPositionerGrps(FunctorParams *functorParams)
                 (std::find(params->m_classIds.begin(), params->m_classIds.end(), positioner->GetObject()->GetClassId())
                     != params->m_classIds.end())
                 && (positioner->GetObject()->GetDrawingGrpId() != 0)
-                && (positioner->GetDrawingPlace() == params->m_place));
+                && (positioner->GetDrawingPlace() == params->m_place) && !positioner->HasEmptyBB());
         });
 
     if (positioners.empty()) {
@@ -803,7 +805,7 @@ int StaffAlignment::AdjustFloatingPositionerGrps(FunctorParams *functorParams)
         // The already move them, so the loop below is not necessary.
     }
     else {
-        // Now go through all the positioners again and ajust the YRel with the value of the pair
+        // Now go through all the positioners again and adjust the YRel with the value of the pair
         for (iter = positioners.begin(); iter != positioners.end(); ++iter) {
             int currentGrpId = (*iter)->GetObject()->GetDrawingGrpId();
             auto i = std::find_if(grpIdYRel.begin(), grpIdYRel.end(),
@@ -837,7 +839,7 @@ int StaffAlignment::AdjustSlurs(FunctorParams *functorParams)
     std::vector<FloatingCurvePositioner *> positioners;
     for (FloatingPositioner *positioner : m_floatingPositioners) {
         assert(positioner->GetObject());
-        if (!positioner->GetObject()->Is({ PHRASE, SLUR, TIE })) continue;
+        if (!positioner->GetObject()->Is({ PHRASE, SLUR })) continue;
         Slur *slur = vrv_cast<Slur *>(positioner->GetObject());
         assert(slur);
 
@@ -853,6 +855,9 @@ int StaffAlignment::AdjustSlurs(FunctorParams *functorParams)
         if (adjusted) {
             params->m_adjusted = true;
         }
+        if (slur->IsCrossStaff()) {
+            params->m_crossStaffSlurs = true;
+        }
     }
 
     Staff *staff = GetStaff();
